#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2020, Paul Aiton <@paultaiton>
# Copyright: (c) 2016, Bruno Medina Bolanos Cacho <bruno.medina@microsoft.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = r'''
---
module: azure_rm_manageddisk_info

version_added: "0.1.2"

short_description: Get managed disk facts

description:
    - Get facts for a specific managed disk or all managed disks.

notes:
    - This module was called M(azure.azcollection.azure_rm_managed_disk_facts) before Ansible 2.8. The usage did not change.

options:
    name:
        description:
            - Limit results to a specific managed disk.
        type: str
    resource_group:
        description:
            - Limit results to a specific resource group.
            - Required if I(name) is set
        type: str
    tags:
        description:
            - Limit results by providing a list of tags.
            - Format tags as 'key' or 'key:value'.
        type: list
        elements: str
    managed_by:
        description:
            - Limit results to disks managed by the given VM fqid.
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Bruno Medina (@brusMX)
    - Paul Aiton (@paultaiton)
'''

EXAMPLES = r'''
- name: Get facts for one managed disk
  azure_rm_manageddisk_info:
    name: Testing
    resource_group: myResourceGroup

- name: Get facts for all managed disks
  azure_rm_manageddisk_info:

- name: Get facts for all managed disks managed by a specific vm
  azure_rm_manageddisk_info:
    managed_by: '/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/rgName/Microsoft.Compute/virtualMachines/vmName'

- name: Get facts by tags
  azure_rm_manageddisk_info:
    tags:
      - testing
'''

RETURN = r'''
azure_managed_disk:
    description:
        - List of managed disk dicts.
    returned: always
    type: list
    contains:
        id:
            description:
                - Resource id.
            type: str
        name:
            description:
                - Name of the managed disk.
            type: str
        location:
            description:
                - Valid Azure location.
            type: str
        storage_account_type:
            description:
                - Type of storage for the managed disk.
                - See U(https://docs.microsoft.com/en-us/azure/virtual-machines/windows/disks-types) for more information about this type.
            type: str
            sample: Standard_LRS
        create_option:
            description:
                - Create option of the disk.
            type: str
            sample: copy
        source_uri:
            description:
                - URI to a valid VHD file to be used or the resource ID of the managed disk to copy.
            type: str
        os_type:
            description:
                - Type of Operating System.
            choices:
                - linux
                - windows
            type: str
        disk_size_gb:
            description:
                - Size in GB of the managed disk to be created.
            type: str
        managed_by:
            description:
                - Name of an existing virtual machine with which the disk is or will be associated, this VM should be in the same resource group.
            type: str
            sample: "/subscriptions/xxx-xxx/resourceGroups/myRG/providers/Microsoft.Compute/virtualMachines/testVM"
        max_shares:
            description:
                - The maximum number of VMs that can attach to the disk at the same time.
                - Value greater than one indicates a disk that can be mounted on multiple VMs at the same time.
            type: int
            sample: 3
        managed_by_extended:
            description:
                - List ID of an existing virtual machine with which the disk is or will be associated.
            type: list
            sample: ["/subscriptions/xxx-xxx/resourceGroups/myRG/providers/Microsoft.Compute/virtualMachines/testVM"]
        tags:
            description:
                - Tags to assign to the managed disk.
            type: dict
            sample: { "tag": "value" }
        tier:
            description:
                - Performance tier assigned to the managed disk.
                - See U(https://learn.microsoft.com/en-us/azure/virtual-machines/disks-change-performance) for more information about disk performance tiers.
            type: str
            sample: "P30"
        time_created:
            description:
                - The time the disk was created.
            type: str
            sample: "2018-01-01T11:08:15.338648900:00"
        disk_iops_read_write:
            description:
                - The number of IOPS allowed for this disk.
                - Only settable for I(storage_account_type=UltraSSD_LRS) disks.
                - One operation can transfer between 4k and 256k bytes.
            type: int
            returned: always
            sample: 200
        disk_m_bps_read_write:
            description:
                - The bandwidth allowed for this disk.
                - Only settable for I(storage_account_type=UltraSSD_LRS) disks.
                - One operation can transfer between 4k and 256k bytes.
            type: int
            returned: always
            sample: 30
        disk_iops_read_only:
            description:
                - The total throughput (MBps) that will be allowed across all VMs mounting the shared disk as ReadOnly.
                - One operation can transfer between 4k and 256k bytes.
            type: int
            returned: always
            sample: 200
        disk_m_bps_read_only:
            description:
                - The total throughput (MBps) that will be allowed across all VMs mounting the shared disk as ReadOnly.
                - MBps means millions of bytes per second - MB here uses the ISO notation, of powers of 10.
            type: int
            returned: always
            sample: 30
        network_access_policy:
            description:
                - Policy for accessing the disk via network.
            type: str
            returned: always
            sample: AllowAll
        public_network_access:
            description:
                - Policy for controlling export on the disk.
            type: str
            returned: always
            sample: Enabled
        disk_access_id:
            description:
                - ARM ID of the DiskAccess resource for using private endpoints on disks.
            type: str
            returned: always
            sample: '/subscriptions/*********/resourceGroups/myRG/providers/Microsoft.Compute/diskAccesses/diskacc'
        performance_plus:
            description:
                - The flag of the performance target of the disk deployed.
            type: bool
            returned: always
            sample: False
        upload_size_bytes:
            description:
                - This is the size of the contents of the upload including the VHD footer.
            type: int
            returned: always
            sample: None
        gallery_image_reference:
            description:
                - The Gallery Image info.
            type: dict
            returned: always
            sample: None
        image_reference:
            description:
                - Disk source information for PIR or user images or Gallery Image.
            type: dict
            returned: always
            sample: None
        logical_sector_size:
            description:
                - Logical sector size in bytes for Ultra disks.
            type: int
            returned: always
            sample: None
        last_ownership_update_time:
            description:
                - The UTC time when the ownership state of the disk was last changed.
                - The time the disk was last attached or detached from a VM or the time when the VM to which the disk was attached was deallocated or started.
            returned: always
            type: str
            sample: "2025-06-27T01:55:10.239311+00:00"
        source_resource_id:
            description:
                - This is the ARM id of the source snapshot or disk.
            type: str
            returned: always
            sample: None
        security_profile:
            description:
                - The security related information for the resource.
            type: complex
            contains:
                security_type:
                    description:
                        - Specifies the SecurityType of the VM.
                    type: str
                    returned: when-used
                    sample: TrustedLaunch
                secure_vm_disk_encryption_set_id:
                    description:
                        -  ResourceId of the disk encryption set associated to Confidential VM supported disk encrypted with customer managed key.
                    type: str
                    returned: when-used
                    sample: None

'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
except Exception:
    # handled in azure_rm_common
    pass


class AzureRMManagedDiskInfo(AzureRMModuleBase):
    """Utility class to get managed disk facts"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(type='str'),
            name=dict(type='str'),
            tags=dict(type='list', elements='str'),
            managed_by=dict(type='str')
        )

        self.results = dict(
            ansible_info=dict(
                azure_managed_disk=[]
            )
        )

        self.resource_group = None
        self.name = None
        self.tags = None
        self.managed_by = None

        super(AzureRMManagedDiskInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                     supports_check_mode=True,
                                                     facts_module=True,
                                                     supports_tags=False)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name and not self.resource_group:
            self.fail('Parameter Error: name requires that resource_group also be set.')

        if self.name:
            self.results['ansible_info']['azure_managed_disk'] = self.get_disk()
        elif self.resource_group:
            self.results['ansible_info']['azure_managed_disk'] = self.list_disks_by_resource_group()
        else:
            self.results['ansible_info']['azure_managed_disk'] = self.list_disks()

        return self.results

    def get_disk(self):
        """Get a single managed disk"""
        results = []

        try:
            results = [self.disk_client.disks.get(self.resource_group,
                                                  self.name)]
            if self.managed_by:
                results = [disk for disk in results if disk.managed_by == self.managed_by]
            if self.tags:
                results = [disk for disk in results if self.has_tags(disk.tags, self.tags)]
            results = [self.managed_disk_to_dict(disk) for disk in results]
        except ResourceNotFoundError:
            self.log('Could not find disk {0} in resource group {1}'.format(self.name, self.resource_group))

        return results

    def list_disks(self):
        """Get all managed disks"""
        results = []

        try:
            results = self.disk_client.disks.list()
            if self.managed_by:
                results = [disk for disk in results if disk.managed_by == self.managed_by]
            if self.tags:
                results = [disk for disk in results if self.has_tags(disk.tags, self.tags)]
            results = [self.managed_disk_to_dict(disk) for disk in results]
        except ResourceNotFoundError as exc:
            self.fail('Failed to list all items - {0}'.format(str(exc)))

        return results

    def list_disks_by_resource_group(self):
        """Get managed disks in a resource group"""
        results = []

        try:
            results = self.disk_client.disks.list_by_resource_group(resource_group_name=self.resource_group)
            if self.managed_by:
                results = [disk for disk in results if disk.managed_by == self.managed_by]
            if self.tags:
                results = [disk for disk in results if self.has_tags(disk.tags, self.tags)]
            results = [self.managed_disk_to_dict(disk) for disk in results]
        except ResourceNotFoundError as exc:
            self.fail('Failed to list items by resource group - {0}'.format(str(exc)))

        return results

    def managed_disk_to_dict(self, managed_disk):
        create_data = managed_disk.creation_data
        return dict(
            id=managed_disk.id,
            name=managed_disk.name,
            location=managed_disk.location,
            tags=managed_disk.tags,
            create_option=create_data.create_option.lower(),
            source_uri=create_data.source_uri or create_data.source_resource_id,
            disk_size_gb=managed_disk.disk_size_gb,
            os_type=managed_disk.os_type.lower() if managed_disk.os_type else None,
            storage_account_type=managed_disk.sku.name if managed_disk.sku else None,
            managed_by=managed_disk.managed_by,
            max_shares=managed_disk.max_shares,
            managed_by_extended=managed_disk.managed_by_extended,
            zone=managed_disk.zones[0] if managed_disk.zones and len(managed_disk.zones) > 0 else '',
            time_created=managed_disk.time_created.isoformat() if managed_disk.time_created else None,
            disk_iops_read_write=managed_disk.disk_iops_read_write,
            disk_m_bps_read_write=managed_disk.disk_m_bps_read_write,
            disk_iops_read_only=managed_disk.disk_iops_read_only,
            disk_m_bps_read_only=managed_disk.disk_m_bps_read_only,
            tier=managed_disk.tier,
            network_access_policy=managed_disk.network_access_policy,
            public_network_access=managed_disk.public_network_access,
            disk_access_id=managed_disk.disk_access_id,
            source_resource_id=create_data.source_resource_id,
            storage_account_id=create_data.storage_account_id,
            upload_size_bytes=create_data.upload_size_bytes,
            logical_sector_size=create_data.logical_sector_size,
            performance_plus=create_data.performance_plus,
            last_ownership_update_time=managed_disk.last_ownership_update_time,
            gallery_image_reference=dict(
                id=create_data.gallery_image_reference.id,
                shared_gallery_image_id=create_data.gallery_image_reference.shared_gallery_image_id,
                community_gallery_image_id=create_data.gallery_image_reference.community_gallery_image_id
            ) if create_data.gallery_image_reference is not None else None,
            image_reference=dict(
                id=create_data.image_reference.id,
                shared_gallery_image_id=create_data.image_reference.shared_gallery_image_id,
                community_gallery_image_id=create_data.image_reference.community_gallery_image_id
            ) if create_data.image_reference is not None else None,
            security_profile=dict(
                security_type=managed_disk.security_profile.security_type,
                secure_vm_disk_encryption_set_id=managed_disk.security_profile.secure_vm_disk_encryption_set_id
            ) if managed_disk.security_profile is not None else None
        )


def main():
    """Main module execution code path"""
    AzureRMManagedDiskInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Shreyas Srish (@shrsr) <ssrish@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = r"""
---
module: ndo_ipsla_monitoring_policy
short_description: Manage IPSLA Monitoring Policies on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage IPSLA Monitoring Policies on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Shreyas Srish (@shrsr)
options:
  template:
    description:
    - The name of the template.
    - The template must be a tenant template.
    type: str
    required: true
  ipsla_monitoring_policy:
    description:
    - The name of the IPSLA Monitoring Policy.
    type: str
    aliases: [ name ]
  description:
    description:
    - The description of the IPSLA Monitoring Policy.
    type: str
  ipsla_monitoring_policy_uuid:
    description:
    - The UUID of the IPSLA Monitoring Policy.
    - This parameter is required when the O(ipsla_monitoring_policy) attribute needs to be updated.
    type: str
    aliases: [ uuid ]
  sla_type:
    description:
    - The type of SLA.
    - The default value is C(icmp).
    type: str
    choices: [ http, tcp, icmp, l2ping ]
  destination_port:
    description:
    - The port used for SLA.
    - This is only applicable when O(sla_type=tcp) or O(sla_type=http).
    - The value defaults to 80 when O(sla_type=http).
    - The value must be in the range 1 - 65535 when O(sla_type=tcp).
    type: int
  http_version:
    description:
    - The HTTP version used for SLA.
    - This is only applicable when O(sla_type=http).
    - The default value is C(HTTP10).
    type: str
    choices: [ HTTP10, HTTP11 ]
  http_uri:
    description:
    - The URI used for HTTP SLA.
    - This is only applicable when O(sla_type=http).
    - The default value is '/'.
    type: str
  sla_frequency:
    description:
    - The frequency of SLA monitoring in seconds.
    - The value must be in the range 1 - 300.
    - The default value is 60 seconds.
    type: int
  detect_multiplier:
    description:
    - The detection multiplier for SLA.
    - The value must be in the range 1 - 100.
    - The default value is 3.
    type: int
  request_data_size:
    description:
    - The size of the request data in bytes.
    - The value must be in the range 0 - 17513.
    - The default value is 28 bytes.
    type: int
  type_of_service:
    description:
    - The IPv4 Type of Service.
    - The value must be in the range 0 - 255.
    - The default value is 0.
    type: int
  operation_timeout:
    description:
    - The operation_timeout for SLA in milliseconds.
    - The default value is 900 milliseconds.
    type: int
  threshold:
    description:
    - The threshold for SLA in milliseconds.
    - The default value is 900 milliseconds.
    type: int
  ipv6_traffic_class:
    description:
    - The IPv6 Traffic Class.
    - The value must be in the range 0 - 255.
    - The default value is 0.
    type: int
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a new IPSLA monitoring policy
  cisco.mso.ndo_ipsla_monitoring_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    ipsla_monitoring_policy: ansible_test_ipsla_monitoring_policy
    sla_type: http
    http_version: HTTP11
    http_uri: /test
    sla_frequency: 120
    detect_multiplier: 5
    request_data_size: 64
    type_of_service: 16
    operation_timeout: 1000
    threshold: 1000
    ipv6_traffic_class: 32
    state: present

- name: Query an IPSLA monitoring policy with name
  cisco.mso.ndo_ipsla_monitoring_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    ipsla_monitoring_policy: ansible_test_ipsla_monitoring_policy
    state: query
  register: query_one

- name: Query all IPSLA monitoring policies in the template
  cisco.mso.ndo_ipsla_monitoring_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    state: query
  register: query_all

- name: Delete an IPSLA monitoring policy
  cisco.mso.ndo_ipsla_monitoring_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    ipsla_monitoring_policy: ansible_test_ipsla_monitoring_policy
    state: absent

- name: Query an IPSLA monitoring policy with UUID
  cisco.mso.ndo_ipsla_monitoring_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    ipsla_monitoring_policy_uuid: "{{ query_one.current.uuid }}"
    state: query
  register: query_one_uuid

- name: Delete an IPSLA monitoring policy with UUID
  cisco.mso.ndo_ipsla_monitoring_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    ipsla_monitoring_policy_uuid: "{{ query_one.current.uuid }}"
    state: absent
"""

RETURN = r"""
"""

import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import (
    MSOModule,
    mso_argument_spec,
)
from ansible_collections.cisco.mso.plugins.module_utils.template import (
    MSOTemplate,
    KVPair,
)
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str", required=True),
        ipsla_monitoring_policy=dict(type="str", aliases=["name"]),
        description=dict(type="str"),
        ipsla_monitoring_policy_uuid=dict(type="str", aliases=["uuid"]),
        sla_type=dict(type="str", choices=["http", "tcp", "icmp", "l2ping"]),
        destination_port=dict(type="int"),
        http_version=dict(type="str", choices=["HTTP10", "HTTP11"]),
        http_uri=dict(type="str"),
        sla_frequency=dict(type="int"),
        detect_multiplier=dict(type="int"),
        request_data_size=dict(type="int"),
        type_of_service=dict(type="int"),
        operation_timeout=dict(type="int"),
        threshold=dict(type="int"),
        ipv6_traffic_class=dict(type="int"),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["ipsla_monitoring_policy", "ipsla_monitoring_policy_uuid"], True],
            ["state", "present", ["ipsla_monitoring_policy", "ipsla_monitoring_policy_uuid"], True],
        ],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    ipsla_monitoring_policy = module.params.get("ipsla_monitoring_policy")
    description = module.params.get("description")
    ipsla_monitoring_policy_uuid = module.params.get("ipsla_monitoring_policy_uuid")
    sla_type = module.params.get("sla_type")
    destination_port = module.params.get("destination_port")
    http_version = module.params.get("http_version")
    http_uri = module.params.get("http_uri")
    sla_frequency = module.params.get("sla_frequency")
    detect_multiplier = module.params.get("detect_multiplier")
    request_data_size = module.params.get("request_data_size")
    type_of_service = module.params.get("type_of_service")
    operation_timeout = module.params.get("operation_timeout")
    threshold = module.params.get("threshold")
    ipv6_traffic_class = module.params.get("ipv6_traffic_class")
    state = module.params.get("state")

    if sla_frequency is not None and sla_frequency not in range(1, 301):
        mso.fail_json(msg="Invalid value provided for sla_frequency: {0}; it must be in the range 1 - 300".format(sla_frequency))

    if detect_multiplier is not None and detect_multiplier not in range(1, 101):
        mso.fail_json(msg="Invalid value provided for detect_multiplier: {0}; it must be in the range 1 - 100".format(detect_multiplier))

    if request_data_size is not None and request_data_size not in range(0, 17514):
        mso.fail_json(msg="Invalid value provided for request_data_size: {0}; it must be in the range 0 - 17513".format(request_data_size))

    if type_of_service is not None and type_of_service not in range(0, 256):
        mso.fail_json(msg="Invalid value provided for type_of_service: {0}; it must be in the range 0 - 255".format(type_of_service))

    if operation_timeout is not None and sla_frequency is not None and operation_timeout > (sla_frequency * 1000):
        mso.fail_json(
            msg="Invalid value provided for operation_timeout: {0}; must be less than or equal to: {1}".format(operation_timeout, (sla_frequency * 1000))
        )

    if threshold is not None and operation_timeout is not None and threshold > operation_timeout:
        mso.fail_json(
            msg="Invalid value provided for threshold: {0}; it must be less than or equal to the operation_timeout: {1}".format(threshold, operation_timeout)
        )

    if ipv6_traffic_class is not None and ipv6_traffic_class not in range(0, 256):
        mso.fail_json(msg="Invalid value provided for ipv6_traffic_class: {0}; it must be in the range 0 - 255".format(ipv6_traffic_class))

    if sla_type == "http":
        destination_port = 80

    if sla_type != "http":
        if http_version or http_uri:
            mso.fail_json(msg="http_version and http_uri can only be used when sla_type is 'http'")

    ops = []
    match = None

    mso_template = MSOTemplate(mso, "tenant", template)
    mso_template.validate_template("tenantPolicy")
    object_description = "IPSLA Monitoring Policy"

    path = "/tenantPolicyTemplate/template/ipslaMonitoringPolicies"
    match = mso_template.get_ipsla_monitoring_policy(ipsla_monitoring_policy_uuid, ipsla_monitoring_policy)
    if isinstance(match, list):  # Query all policies
        mso.existing = mso.previous = match
    elif match is not None:  # Query a specific policy
        mso.existing = mso.previous = copy.deepcopy(match.details)

    if state == "present":
        mso_values = dict(
            name=ipsla_monitoring_policy,
            description=description,
            slaType=sla_type,
            slaPort=destination_port,
            httpVersion=http_version,
            httpUri=http_uri,
            slaFrequency=sla_frequency,
            slaDetectMultiplier=detect_multiplier,
            reqDataSize=request_data_size,
            ipv4ToS=type_of_service,
            timeout=operation_timeout,
            threshold=threshold,
            ipv6TrfClass=ipv6_traffic_class,
        )
        if match:
            update_path = "{0}/{1}".format(path, match.index)
            append_update_ops_data(ops, match.details, update_path, mso_values)
            mso.sanitize(match.details, collate=True)

        else:
            mso.sanitize(mso_values)
            ops.append(dict(op="add", path="{0}/-".format(path), value=mso.sent))

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path="{0}/{1}".format(path, match.index)))

    if not module.check_mode and ops:
        response = mso.request(mso_template.template_path, method="PATCH", data=ops)
        ipsla_policies = response.get("tenantPolicyTemplate", {}).get("template", {}).get("ipslaMonitoringPolicies", [])
        match = mso_template.get_object_by_key_value_pairs(
            object_description,
            ipsla_policies,
            [KVPair("uuid", ipsla_monitoring_policy_uuid) if ipsla_monitoring_policy_uuid else KVPair("name", ipsla_monitoring_policy)],
        )
        if match:
            mso.existing = match.details
        else:
            mso.existing = {}
    elif module.check_mode and state != "query":
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2020 Matt Clay <mclay@redhat.com>
# Copyright (c) 2020 Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: current_container_facts
short_description: Return facts about whether the module runs in a container
version_added: 1.1.0
description:
  - Return facts about whether the module runs in a Docker or podman container.
  - This module attempts a best-effort detection. There might be special cases where it does not work; if you encounter one,
    make sure that this is still a problem with the latest community.docker release, and if it is,
    L(please file an issue, https://github.com/ansible-collections/community.docker/issues/new?assignees=&labels=&template=bug_report.md).
author:
  - Felix Fontein (@felixfontein)
extends_documentation_fragment:
  - community.docker._attributes
  - community.docker._attributes.facts
  - community.docker._attributes.facts_module
  - community.docker._attributes.idempotent_not_modify_state
"""

EXAMPLES = r"""
---
- name: Get facts on current container
  community.docker.current_container_facts:

- name: Print information on current container when running in a container
  ansible.builtin.debug:
    msg: "Container ID is {{ ansible_module_container_id }}"
  when: ansible_module_running_in_container
"""

RETURN = r"""
ansible_facts:
  description: Ansible facts returned by the module.
  type: dict
  returned: always
  contains:
    ansible_module_running_in_container:
      description:
        - Whether the module was able to detect that it runs in a container or not.
      returned: always
      type: bool
    ansible_module_container_id:
      description:
        - The detected container ID.
        - Contains an empty string if no container was detected.
      returned: always
      type: str
    ansible_module_container_type:
      description:
        - The detected container environment.
        - Contains an empty string if no container was detected, or a non-empty string identifying the container environment.
        - V(docker) indicates that the module ran inside a regular Docker container.
        - V(azure_pipelines) indicates that the module ran on Azure Pipelines. This seems to no longer be reported.
        - V(github_actions) indicates that the module ran inside a Docker container on GitHub Actions. It is supported since
          community.docker 2.4.0.
        - V(podman) indicates that the module ran inside a regular Podman container. It is supported since community.docker
          3.3.0.
      returned: always
      type: str
      choices:
        - ''
        - docker
        - azure_pipelines
        - github_actions
        - podman
"""

import os
import re

from ansible.module_utils.basic import AnsibleModule


def main() -> None:
    module = AnsibleModule({}, supports_check_mode=True)

    cpuset_path = "/proc/self/cpuset"
    mountinfo_path = "/proc/self/mountinfo"

    container_id = ""
    container_type = ""

    contents = None
    if os.path.exists(cpuset_path):
        # File content varies based on the environment:
        #   No Container: /
        #   Docker: /docker/c86f3732b5ba3d28bb83b6e14af767ab96abbc52de31313dcb1176a62d91a507
        #   Azure Pipelines (Docker): /azpl_job/0f2edfed602dd6ec9f2e42c867f4d5ee640ebf4c058e6d3196d4393bb8fd0891
        #   Podman: /../../../../../..
        # While this was true and worked well for a long time, this seems to be no longer accurate
        # with newer Docker / Podman versions and/or with cgroupv2. That's why the /proc/self/mountinfo
        # detection further down is done when this test is inconclusive.
        with open(cpuset_path, "rb") as f:
            contents = f.read().decode("utf-8")

        cgroup_path, cgroup_name = os.path.split(contents.strip())

        if cgroup_path == "/docker":
            container_id = cgroup_name
            container_type = "docker"

        if cgroup_path == "/azpl_job":
            container_id = cgroup_name
            container_type = "azure_pipelines"

        if cgroup_path == "/actions_job":
            container_id = cgroup_name
            container_type = "github_actions"

    if not container_id and os.path.exists(mountinfo_path):
        with open(mountinfo_path, "rb") as f:
            contents = f.read().decode("utf-8")

        # As to why this works, see the explanations by Matt Clay in
        # https://github.com/ansible/ansible/blob/80d2f8da02052f64396da6b8caaf820eedbf18e2/test/lib/ansible_test/_internal/docker_util.py#L571-L610

        for line in contents.splitlines():
            parts = line.split()
            if len(parts) >= 5 and parts[4] == "/etc/hostname":
                m = re.match(".*/([a-f0-9]{64})/hostname$", parts[3])
                if m:
                    container_id = m.group(1)
                    container_type = "docker"

                m = re.match(".*/([a-f0-9]{64})/userdata/hostname$", parts[3])
                if m:
                    container_id = m.group(1)
                    container_type = "podman"

    module.exit_json(
        ansible_facts={
            "ansible_module_running_in_container": container_id != "",
            "ansible_module_container_id": container_id,
            "ansible_module_container_type": container_type,
        }
    )


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2018 Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: acme_ari_info
author: "Felix Fontein (@felixfontein)"
version_added: 2.20.0
short_description: Retrieves ACME Renewal Information (ARI) for a certificate
description:
  - Allows to retrieve renewal information on a certificate obtained with the L(ACME protocol,https://tools.ietf.org/html/rfc8555).
  - This module only works with the ACME v2 protocol, and requires the ACME server to support the ARI extension
    (L(RFC 9773, https://www.rfc-editor.org/rfc/rfc9773.html)).
extends_documentation_fragment:
  - community.crypto._acme.basic
  - community.crypto._acme.no_account
  - community.crypto._attributes
  - community.crypto._attributes.info_module
  - community.crypto._attributes.idempotent_not_modify_state
options:
  certificate_path:
    description:
      - A path to the X.509 certificate to request information for.
      - Exactly one of O(certificate_path) and O(certificate_content) must be provided.
    type: path
  certificate_content:
    description:
      - The content of the X.509 certificate to request information for.
      - Exactly one of O(certificate_path) and O(certificate_content) must be provided.
    type: str
seealso:
  - module: community.crypto.acme_certificate
    description: Allows to obtain a certificate using the ACME protocol.
  - module: community.crypto.acme_certificate_revoke
    description: Allows to revoke a certificate using the ACME protocol.
"""

EXAMPLES = r"""
---
- name: Retrieve renewal information for a certificate
  community.crypto.acme_ari_info:
    certificate_path: /etc/httpd/ssl/sample.com.crt
  register: cert_data

- name: Show the certificate renewal information
  ansible.builtin.debug:
    var: cert_data.renewal_info
"""

RETURN = r"""
renewal_info:
  description: The ARI renewal info object (U(https://www.rfc-editor.org/rfc/rfc9773.html#section-4.2)).
  returned: success
  type: dict
  contains:
    suggestedWindow:
      description:
        - Describes the window during which the certificate should be renewed.
      type: dict
      returned: always
      contains:
        start:
          description:
            - The start of the window during which the certificate should be renewed.
            - The format is specified in L(RFC 3339,https://www.rfc-editor.org/info/rfc3339).
          returned: always
          type: str
          sample: '2021-01-03T00:00:00Z'
        end:
          description:
            - The end of the window during which the certificate should be renewed.
            - The format is specified in L(RFC 3339,https://www.rfc-editor.org/info/rfc3339).
          returned: always
          type: str
          sample: '2021-01-03T00:00:00Z'
    explanationURL:
      description:
        - A URL pointing to a page which may explain why the suggested renewal window is what it is.
        - For example, it may be a page explaining the CA's dynamic load-balancing strategy, or a page documenting which certificates
          are affected by a mass revocation event. Should be shown to the user.
      returned: depends on the ACME server
      type: str
      sample: https://example.com/docs/ari
    retryAfter:
      description:
        - A timestamp before the next retry to ask for this information should not be made.
      returned: depends on the ACME server
      type: str
      sample: '2024-04-29T01:17:10.236921+00:00'
"""

import typing as t

from ansible_collections.community.crypto.plugins.module_utils._acme.acme import (
    ACMEClient,
    create_backend,
    create_default_argspec,
)
from ansible_collections.community.crypto.plugins.module_utils._acme.errors import (
    ModuleFailException,
)


def main() -> t.NoReturn:
    argument_spec = create_default_argspec(with_account=False)
    argument_spec.update_argspec(
        certificate_path={"type": "path"},
        certificate_content={"type": "str"},
    )
    argument_spec.update(
        required_one_of=[("certificate_path", "certificate_content")],
        mutually_exclusive=[("certificate_path", "certificate_content")],
    )
    module = argument_spec.create_ansible_module(supports_check_mode=True)
    backend = create_backend(module, needs_acme_v2=True)

    try:
        client = ACMEClient(module=module, backend=backend)
        if not client.directory.has_renewal_info_endpoint():
            module.fail_json(
                msg="The ACME endpoint does not support ACME Renewal Information retrieval"
            )
        renewal_info = client.get_renewal_info(
            cert_filename=module.params["certificate_path"],
            cert_content=module.params["certificate_content"],
            include_retry_after=True,
        )
        module.exit_json(renewal_info=renewal_info)
    except ModuleFailException as e:
        e.do_fail(module=module)


if __name__ == "__main__":
    main()



# Copyright (c) 2024, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

"""
Parse go logfmt messages.

See https://pkg.go.dev/github.com/kr/logfmt?utm_source=godoc for information on the format.
"""

from __future__ import annotations

import typing as t
from enum import Enum

# The format is defined in https://pkg.go.dev/github.com/kr/logfmt?utm_source=godoc
# (look for "EBNFish")


class InvalidLogFmt(Exception):
    pass


class _Mode(Enum):
    GARBAGE = 0
    KEY = 1
    EQUAL = 2
    IDENT_VALUE = 3
    QUOTED_VALUE = 4


_ESCAPE_DICT = {
    '"': '"',
    "\\": "\\",
    "'": "'",
    "/": "/",
    "b": "\b",
    "f": "\f",
    "n": "\n",
    "r": "\r",
    "t": "\t",
}

_HEX_DICT = {
    "0": 0,
    "1": 1,
    "2": 2,
    "3": 3,
    "4": 4,
    "5": 5,
    "6": 6,
    "7": 7,
    "8": 8,
    "9": 9,
    "a": 0xA,
    "b": 0xB,
    "c": 0xC,
    "d": 0xD,
    "e": 0xE,
    "f": 0xF,
    "A": 0xA,
    "B": 0xB,
    "C": 0xC,
    "D": 0xD,
    "E": 0xE,
    "F": 0xF,
}


def _is_ident(cur: str) -> bool:
    return cur > " " and cur not in ('"', "=")


class _Parser:
    def __init__(self, line: str) -> None:
        self.line = line
        self.index = 0
        self.length = len(line)

    def done(self) -> bool:
        return self.index >= self.length

    def cur(self) -> str:
        return self.line[self.index]

    def next(self) -> None:
        self.index += 1

    def prev(self) -> None:
        self.index -= 1

    def parse_unicode_sequence(self) -> str:
        if self.index + 6 > self.length:
            raise InvalidLogFmt("Not enough space for unicode escape")
        if self.line[self.index : self.index + 2] != "\\u":
            raise InvalidLogFmt("Invalid unicode escape start")
        v = 0
        for dummy_index in range(self.index + 2, self.index + 6):
            v <<= 4
            try:
                v += _HEX_DICT[self.line[self.index]]
            except KeyError:
                raise InvalidLogFmt(
                    f"Invalid unicode escape digit {self.line[self.index]!r}"
                ) from None
        self.index += 6
        return chr(v)


def parse_line(line: str, logrus_mode: bool = False) -> dict[str, t.Any]:
    result: dict[str, t.Any] = {}
    parser = _Parser(line)
    key: list[str] = []
    value: list[str] = []
    mode = _Mode.GARBAGE

    def handle_kv(has_no_value: bool = False) -> None:
        k = "".join(key)
        v = None if has_no_value else "".join(value)
        result[k] = v
        del key[:]
        del value[:]

    def parse_garbage(cur: str) -> _Mode:
        if _is_ident(cur):
            return _Mode.KEY
        parser.next()
        return _Mode.GARBAGE

    def parse_key(cur: str) -> _Mode:
        if _is_ident(cur):
            key.append(cur)
            parser.next()
            return _Mode.KEY
        if cur == "=":
            parser.next()
            return _Mode.EQUAL
        if logrus_mode:
            raise InvalidLogFmt('Key must always be followed by "=" in logrus mode')
        handle_kv(has_no_value=True)
        parser.next()
        return _Mode.GARBAGE

    def parse_equal(cur: str) -> _Mode:
        if _is_ident(cur):
            value.append(cur)
            parser.next()
            return _Mode.IDENT_VALUE
        if cur == '"':
            parser.next()
            return _Mode.QUOTED_VALUE
        handle_kv()
        parser.next()
        return _Mode.GARBAGE

    def parse_ident_value(cur: str) -> _Mode:
        if _is_ident(cur):
            value.append(cur)
            parser.next()
            return _Mode.IDENT_VALUE
        handle_kv()
        parser.next()
        return _Mode.GARBAGE

    def parse_quoted_value(cur: str) -> _Mode:
        if cur == "\\":
            parser.next()
            if parser.done():
                raise InvalidLogFmt("Unterminated escape sequence in quoted string")
            cur = parser.cur()
            if cur in _ESCAPE_DICT:
                value.append(_ESCAPE_DICT[cur])
            elif cur != "u":
                es = f"\\{cur}"
                raise InvalidLogFmt(f"Unknown escape sequence {es!r}")
            else:
                parser.prev()
                value.append(parser.parse_unicode_sequence())
            parser.next()
            return _Mode.QUOTED_VALUE
        if cur == '"':
            handle_kv()
            parser.next()
            return _Mode.GARBAGE
        if cur < " ":
            raise InvalidLogFmt("Control characters in quoted string are not allowed")
        value.append(cur)
        parser.next()
        return _Mode.QUOTED_VALUE

    parsers = {
        _Mode.GARBAGE: parse_garbage,
        _Mode.KEY: parse_key,
        _Mode.EQUAL: parse_equal,
        _Mode.IDENT_VALUE: parse_ident_value,
        _Mode.QUOTED_VALUE: parse_quoted_value,
    }
    while not parser.done():
        mode = parsers[mode](parser.cur())
    if mode == _Mode.KEY and logrus_mode:
        raise InvalidLogFmt('Key must always be followed by "=" in logrus mode')
    if mode in (_Mode.KEY, _Mode.EQUAL):
        handle_kv(has_no_value=True)
    elif mode == _Mode.IDENT_VALUE:
        handle_kv()
    elif mode == _Mode.QUOTED_VALUE:
        raise InvalidLogFmt("Unterminated quoted string")
    return result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2021, Kevin Breit (@kbreit) <kevin.breit@kevinbreit.net>
# GNU General Public License v3.0+ (see COPYING or
# https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["deprecated"],
    "supported_by": "community",
}

DOCUMENTATION = r"""
author:
  - Kevin Breit (@kbreit)
deprecated:
  alternative: cisco.meraki.organizations_action_batches
  removed_in: 3.0.0
  why: Updated modules released with increased functionality
description:
  - Allows for management of Action Batch jobs for Meraki.
extends_documentation_fragment: cisco.meraki.meraki
module: meraki_action_batch
notes:
  - This module is in active development and the interface may change.
options:
  action_batch_id:
    description:
      - ID of an existing Action Batch job.
    type: str
  actions:
    description:
      - List of actions the job should execute.
    elements: dict
    suboptions:
      body:
        description:
          - Required body of action.
        type: raw
      operation:
        choices:
          - create
          - destroy
          - update
          - claim
          - bind
          - split
          - unbind
          - combine
          - update_order
          - cycle
          - swap
          - assignSeats
          - move
          - moveSeats
          - renewSeats
        description:
          - Operation type of action
        type: str
      resource:
        description:
          - Path to Action Batch resource.
        type: str
    type: list
  confirmed:
    default: false
    description:
      - Whether job is to be executed.
    type: bool
  net_id:
    description:
      - ID of network, if applicable.
    type: str
  net_name:
    description:
      - Name of network, if applicable.
    type: str
  state:
    choices:
      - query
      - present
      - absent
    default: present
    description:
      - Specifies whether to lookup, create, or delete an Action Batch job.
    type: str
  synchronous:
    default: true
    description:
      - Whether job is a synchronous or asynchronous job.
    type: bool
short_description: Manage Action Batch jobs within the Meraki Dashboard.
"""


EXAMPLES = r"""
- name: Query all Action Batches
  meraki_action_batch:
    auth_key: abc123
    org_name: YourOrg
    state: query
  delegate_to: localhost
- name: Query one Action Batch job
  meraki_action_batch:
    auth_key: abc123
    org_name: YourOrg
    state: query
    action_batch_id: 12345
  delegate_to: localhost
- name: Create an Action Batch job
  meraki_action_batch:
    auth_key: abc123
    org_name: YourOrg
    state: present
    actions:
      - resource: /organizations/org_123/networks
        operation: create
        body:
          name: AnsibleActionBatch1
          productTypes:
            - switch
  delegate_to: localhost
- name: Update Action Batch job
  meraki_action_batch:
    auth_key: abc123
    org_name: YourOrg
    state: present
    action_batch_id: 12345
    synchronous: false
- name: Create an Action Batch job with multiple actions
  meraki_action_batch:
    auth_key: abc123
    org_name: YourOrg
    state: present
    actions:
      - resource: /organizations/org_123/networks
        operation: create
        body:
          name: AnsibleActionBatch2
          productTypes:
            - switch
      - resource: /organizations/org_123/networks
        operation: create
        body:
          name: AnsibleActionBatch3
          productTypes:
            - switch
  delegate_to: localhost
- name: Delete an Action Batch job
  meraki_action_batch:
    auth_key: abc123
    org_name: YourOrg
    state: absent
    action_batch_id: 12345
  delegate_to: localhost
"""

RETURN = r"""
data:
    description: Information about action batch jobs.
    type: complex
    returned: always
    contains:
        id:
            description: Unique ID of action batch job.
            returned: success
            type: str
            sample: 123
        organization_id:
            description: Unique ID of organization which owns batch job.
            returned: success
            type: str
            sample: 2930418
        confirmed:
            description: Whether action batch job was confirmed for execution.
            returned: success
            type: bool
        synchronous:
            description: Whether action batch job executes synchronously or asynchronously.
            returned: success
            type: bool
        status:
            description: Information about the action batch job state.
            type: complex
            contains:
                completed:
                    description: Whether job has completed.
                    type: bool
                    returned: success
                failed:
                    description: Whether execution of action batch job failed.
                    type: bool
                    returned: success
                errors:
                    description: List of errors, if any, created during execution.
                    type: list
                    returned: success
                created_resources:
                    description: List of resources created during execution.
                    type: list
                    returned: success
                    sample: [{"id": 100, "uri": "/networks/L_XXXXX/groupPolicies/100"}]
        actions:
            description: List of actions associated to job.
            type: dict
"""

from ansible_collections.cisco.meraki.plugins.module_utils.network.meraki.meraki import (
    MerakiModule, meraki_argument_spec, )
from ansible.module_utils.basic import AnsibleModule, json


def _construct_payload(meraki):
    payload = dict()
    payload["confirmed"] = meraki.params["confirmed"]
    payload["synchronous"] = meraki.params["synchronous"]
    if meraki.params["actions"] is not None:  # No payload is specified for an update
        payload["actions"] = list()
        for action in meraki.params["actions"]:
            action_detail = dict()
            if action["resource"] is not None:
                action_detail["resource"] = action["resource"]
            if action["operation"] is not None:
                action_detail["operation"] = action["operation"]
            if action["body"] is not None:
                action_detail["body"] = action["body"]
            payload["actions"].append(action_detail)
    return payload


def main():

    # define the available arguments/parameters that a user can pass to
    # the module

    actions_arg_spec = dict(
        operation=dict(
            type="str",
            choices=[
                "create",
                "destroy",
                "update",
                "claim",
                "bind",
                "split",
                "unbind",
                "combine",
                "update_order",
                "cycle",
                "swap",
                "assignSeats",
                "move",
                "moveSeats",
                "renewSeats",
            ],
        ),
        resource=dict(type="str"),
        body=dict(type="raw"),
    )

    argument_spec = meraki_argument_spec()
    argument_spec.update(
        state=dict(
            type="str", choices=["present", "query", "absent"], default="present"
        ),
        net_name=dict(type="str"),
        net_id=dict(type="str"),
        action_batch_id=dict(type="str", default=None),
        confirmed=dict(type="bool", default=False),
        synchronous=dict(type="bool", default=True),
        actions=dict(
            type="list", default=None, elements="dict", options=actions_arg_spec
        ),
    )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )
    meraki = MerakiModule(module, function="action_batch")
    meraki.params["follow_redirects"] = "all"

    query_urls = {"action_batch": "/organizations/{org_id}/actionBatches"}
    query_one_urls = {
        "action_batch": "/organizations/{org_id}/actionBatches/{action_batch_id}"}
    create_urls = {"action_batch": "/organizations/{org_id}/actionBatches"}
    update_urls = {
        "action_batch": "/organizations/{org_id}/actionBatches/{action_batch_id}"}
    delete_urls = {
        "action_batch": "/organizations/{org_id}/actionBatches/{action_batch_id}"}

    meraki.url_catalog["get_all"].update(query_urls)
    meraki.url_catalog["get_one"].update(query_one_urls)
    meraki.url_catalog["create"] = create_urls
    meraki.url_catalog["update"] = update_urls
    meraki.url_catalog["delete"] = delete_urls

    payload = None

    if not meraki.params["org_name"] and not meraki.params["org_id"]:
        meraki.fail_json(msg="org_name or org_id is required")

    org_id = meraki.params["org_id"]
    if org_id is None:
        org_id = meraki.get_org_id(meraki.params["org_name"])

    if meraki.params["state"] == "query":
        if meraki.params["action_batch_id"] is None:  # Get all Action Batches
            path = meraki.construct_path("get_all", org_id=org_id)
            response = meraki.request(path, method="GET")
            if meraki.status == 200:
                meraki.result["data"] = response
                meraki.exit_json(**meraki.result)
        elif meraki.params["action_batch_id"] is not None:  # Query one Action Batch job
            path = meraki.construct_path(
                "get_one",
                org_id=org_id,
                custom={"action_batch_id": meraki.params["action_batch_id"]},
            )
            response = meraki.request(path, method="GET")
            if meraki.status == 200:
                meraki.result["data"] = response
                meraki.exit_json(**meraki.result)
    elif meraki.params["state"] == "present":
        if meraki.params["action_batch_id"] is None:  # Create a new Action Batch job
            payload = _construct_payload(meraki)
            path = meraki.construct_path("create", org_id=org_id)
            response = meraki.request(
                path, method="POST", payload=json.dumps(payload))
            if meraki.status == 201:
                meraki.result["data"] = response
                meraki.result["changed"] = True
                meraki.exit_json(**meraki.result)
        elif meraki.params["action_batch_id"] is not None:
            path = meraki.construct_path(
                "get_one",
                org_id=org_id,
                custom={"action_batch_id": meraki.params["action_batch_id"]},
            )
            current = meraki.request(path, method="GET")
            payload = _construct_payload(meraki)
            if (
                meraki.params["actions"] is not None
            ):  # Cannot update the body once a job is submitted
                meraki.fail_json(msg="Body cannot be updated on existing job.")
            if (
                meraki.is_update_required(current, payload) is True
            ):  # Job needs to be modified
                path = meraki.construct_path(
                    "update", org_id=org_id, custom={
                        "action_batch_id": meraki.params["action_batch_id"]}, )
                response = meraki.request(
                    path, method="PUT", payload=json.dumps(payload)
                )
                if meraki.status == 200:
                    meraki.result["data"] = response
                    meraki.result["changed"] = True
                    meraki.exit_json(**meraki.result)
            else:  # Idempotent response
                meraki.result["data"] = current
                meraki.exit_json(**meraki.result)
    elif meraki.params["state"] == "absent":
        path = meraki.construct_path(
            "delete",
            org_id=org_id,
            custom={"action_batch_id": meraki.params["action_batch_id"]},
        )
        response = meraki.request(path, method="DELETE")
        if meraki.status == 204:
            meraki.result["data"] = response
            meraki.result["changed"] = True

    # in the event of a successful module execution, you will want to
    # simple AnsibleModule.exit_json(), passing the key/value results
    meraki.exit_json(**meraki.result)


if __name__ == "__main__":
    main()



# Copyright (c) 2023, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: gpg_fingerprint
short_description: Retrieve a GPG fingerprint from a GPG public or private key
author: Felix Fontein (@felixfontein)
version_added: 2.15.0
description:
  - Takes the content of a private or public GPG key as input and returns its fingerprint.
options:
  _input:
    description:
      - The content of a GPG public or private key.
    type: string
    required: true
requirements:
  - GnuPG (C(gpg) executable)
seealso:
  - plugin: community.crypto.gpg_fingerprint
    plugin_type: lookup
"""

EXAMPLES = r"""
---
- name: Show fingerprint of GPG public key
  ansible.builtin.debug:
    msg: "{{ lookup('file', '/path/to/public_key.gpg') | community.crypto.gpg_fingerprint }}"
"""

RETURN = r"""
_value:
  description:
    - The fingerprint of the provided public or private GPG key.
  type: string
"""

from collections.abc import Callable

from ansible.errors import AnsibleFilterError
from ansible.module_utils.common.text.converters import to_bytes

from ansible_collections.community.crypto.plugins.module_utils._gnupg.cli import (
    GPGError,
    get_fingerprint_from_bytes,
)
from ansible_collections.community.crypto.plugins.plugin_utils._gnupg import (
    PluginGPGRunner,
)


def gpg_fingerprint(gpg_key_content: str | bytes) -> str:
    if not isinstance(gpg_key_content, (str, bytes)):
        raise AnsibleFilterError(
            f"The input for the community.crypto.gpg_fingerprint filter must be a string; got {type(gpg_key_content)} instead"
        )
    try:
        gpg = PluginGPGRunner()
        return get_fingerprint_from_bytes(
            gpg_runner=gpg, content=to_bytes(gpg_key_content)
        )
    except GPGError as exc:
        raise AnsibleFilterError(str(exc)) from exc


class FilterModule:
    """Ansible jinja2 filters"""

    def filters(self) -> dict[str, Callable]:
        return {
            "gpg_fingerprint": gpg_fingerprint,
        }



#!/usr/bin/python

# Copyright (c) Ansible project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: locale_gen
short_description: Creates or removes locales
description:
  - Manages locales in Debian and Ubuntu systems.
author:
  - Augustus Kling (@AugustusKling)
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    type: list
    elements: str
    description:
      - Name and encoding of the locales, such as V(en_GB.UTF-8).
      - Before community.general 9.3.0, this was a string. Using a string still works.
    required: true
  state:
    type: str
    description:
      - Whether the locales shall be present.
    choices: [absent, present]
    default: present

notes:
  - Currently the module is B(only supported for Debian, Ubuntu, and Arch Linux) systems.
  - This module requires the package C(locales) installed in Debian and Ubuntu systems.
  - If C(/etc/locale.gen) exists, the module assumes to be using the B(glibc) mechanism, else it raises an error.
    Support for C(/var/lib/locales/supported.d/) (the V(ubuntu_legacy) mechanism) has been removed in community.general 13.0.0.
  - When using V(glibc) mechanism, it manages locales by editing C(/etc/locale.gen) and running C(locale-gen).
  - Please note that the module asserts the availability of the locale by checking the files C(/usr/share/i18n/SUPPORTED) and
    C(/usr/local/share/i18n/SUPPORTED), but the C(/usr/local) one is not supported by Archlinux.
"""

EXAMPLES = r"""
- name: Ensure a locale exists
  community.general.locale_gen:
    name: de_CH.UTF-8
    state: present

- name: Ensure multiple locales exist
  community.general.locale_gen:
    name:
      - en_GB.UTF-8
      - nl_NL.UTF-8
    state: present
"""

RETURN = r"""
mechanism:
  description: Mechanism used to deploy the locales.
  type: str
  choices:
    - glibc
  returned: success
  sample: glibc
  version_added: 10.2.0
"""

import os
import re

from ansible_collections.community.general.plugins.module_utils._locale_gen import locale_gen_runner, locale_runner
from ansible_collections.community.general.plugins.module_utils._mh.deco import check_mode_skip
from ansible_collections.community.general.plugins.module_utils._module_helper import StateModuleHelper

ETC_LOCALE_GEN = "/etc/locale.gen"
VAR_LIB_LOCALES = "/var/lib/locales/supported.d"
VAR_LIB_LOCALES_LOCAL = os.path.join(VAR_LIB_LOCALES, "local")
SUPPORTED_LOCALES = ["/usr/share/i18n/SUPPORTED", "/usr/local/share/i18n/SUPPORTED"]
RE_LOCALE_ENTRY = re.compile(r"^\s*#?\s*(?P<locale>\S+[\._\S]+) (?P<charset>\S+)\s*$")
LOCALE_NORMALIZATION = {
    ".utf8": ".UTF-8",
    ".eucjp": ".EUC-JP",
    ".iso885915": ".ISO-8859-15",
    ".cp1251": ".CP1251",
    ".koi8r": ".KOI8-R",
    ".armscii8": ".ARMSCII-8",
    ".euckr": ".EUC-KR",
    ".gbk": ".GBK",
    ".gb18030": ".GB18030",
    ".euctw": ".EUC-TW",
}


class LocaleGen(StateModuleHelper):
    output_params = ["name"]
    module = dict(
        argument_spec=dict(
            name=dict(type="list", elements="str", required=True),
            state=dict(type="str", default="present", choices=["absent", "present"]),
        ),
        supports_check_mode=True,
    )

    def __init_module__(self):
        self.mechanisms = dict(
            glibc=dict(
                available=SUPPORTED_LOCALES,
                apply_change=self.apply_change_glibc,
            ),
        )

        if os.path.exists(ETC_LOCALE_GEN):
            self.vars.ubuntu_mode = False
            self.vars.mechanism = "glibc"
        else:
            self.do_raise(f'{ETC_LOCALE_GEN} is missing. Is the package "locales" installed?')

        self.runner = locale_runner(self.module)

        self.assert_available()
        self.vars.set("is_present", self.is_present(), output=False)
        self.vars.set("state_tracking", self._state_name(self.vars.is_present), output=False, change=True)

    def __quit_module__(self):
        self.vars.state_tracking = self._state_name(self.is_present())

    @staticmethod
    def _state_name(present):
        return "present" if present else "absent"

    def assert_available(self):
        """Check if the given locales are available on the system. This is done by
        checking either :
        * if the locale is present in /etc/locales.gen
        * or if the locale is present in /usr/share/i18n/SUPPORTED"""

        self.vars.set("available_lines", [], verbosity=4)
        available_locale_entry_re_matches = []
        for locale_path in self.mechanisms[self.vars.mechanism]["available"]:
            if os.path.exists(locale_path):
                with open(locale_path) as fd:
                    self.vars.available_lines.extend(fd.readlines())

        available_locale_entry_re_matches.extend([RE_LOCALE_ENTRY.match(line) for line in self.vars.available_lines])

        locales_not_found = []
        for locale in self.vars.name:
            # Check if the locale is not found in any of the matches
            if not any(match and match.group("locale") == locale for match in available_locale_entry_re_matches):
                locales_not_found.append(locale)

        # locale may be installed but not listed in the file, for example C.UTF-8 in some systems
        locales_not_found = self.locale_get_not_present(locales_not_found)

        if locales_not_found:
            self.do_raise(
                f"The following locales you have entered are not available on your system: {', '.join(locales_not_found)}"
            )

    def is_present(self):
        return not self.locale_get_not_present(self.vars.name)

    def locale_get_not_present(self, locales):
        runner = locale_runner(self.module)
        with runner() as ctx:
            rc, out, err = ctx.run()
            if self.verbosity >= 4:
                self.vars.locale_run_info = ctx.run_info

        not_found = []
        for locale in locales:
            if not any(self.fix_case(locale) == self.fix_case(line) for line in out.splitlines()):
                not_found.append(locale)

        return not_found

    def fix_case(self, name):
        """locale -a might return the encoding in either lower or upper case.
        Passing through this function makes them uniform for comparisons."""
        for s, r in LOCALE_NORMALIZATION.items():
            name = name.replace(s, r)
        return name

    def _get_charset_from_supported(self, locale):
        """Look up the charset for a locale from the SUPPORTED files."""
        for locale_path in SUPPORTED_LOCALES:
            if os.path.exists(locale_path):
                with open(locale_path) as fd:
                    for line in fd:
                        match = RE_LOCALE_ENTRY.match(line)
                        if match and match.group("locale") == locale:
                            return match.group("charset")
        return None

    def set_locale_glibc(self, names, enabled=True):
        """Sets the state of the locale. Defaults to enabled."""
        with open(ETC_LOCALE_GEN) as fr:
            lines = fr.readlines()

        locale_regexes = {}
        matched = set()

        for name in names:
            search_string = rf"^#?\s*{re.escape(name)} (?P<charset>.+)"
            if enabled:
                new_string = rf"{name} \g<charset>"
            else:
                new_string = rf"# {name} \g<charset>"
            locale_regexes[name] = (re.compile(search_string), new_string)

        def search_replace(line):
            for name, (search, replace) in locale_regexes.items():
                new_line = search.sub(replace, line)
                if new_line != line:
                    matched.add(name)
                line = new_line
            return line

        lines = [search_replace(line) for line in lines]

        # For locales not found in /etc/locale.gen (e.g. on Gentoo), add them
        if enabled:
            for name in names:
                if name not in matched:
                    charset = self._get_charset_from_supported(name)
                    if charset:
                        lines.append(f"{name} {charset}\n")

        # Write the modified content back to the file
        with open(ETC_LOCALE_GEN, "w") as fw:
            fw.writelines(lines)

    def apply_change_glibc(self, target_state, names):
        """Create or remove locale.

        Keyword arguments:
        target_state -- Desired state, either present or absent.
        names -- Names list including encoding such as de_CH.UTF-8.
        """

        self.set_locale_glibc(names, enabled=(target_state == "present"))

        runner = locale_gen_runner(self.module)
        with runner() as ctx:
            ctx.run()

    @check_mode_skip
    def __state_fallback__(self):
        if self.vars.state_tracking == self.vars.state:
            return
        self.mechanisms[self.vars.mechanism]["apply_change"](self.vars.state, self.vars.name)


def main():
    LocaleGen.execute()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for nxos_static_routes
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: nxos_static_routes
short_description: Static routes resource module
description: This module configures and manages the attributes of static routes on
  Cisco NX-OS platforms.
version_added: 1.0.0
author:
  - Adharsh Srivats Rangarajan (@adharshsrivatsr)
  - Sagar Paul (@KB-perByte)
notes:
- Tested against Nexus 9300v running NX-OS 9.3.6 on CML.
- Unsupported for Cisco MDS
- When a route is configured for a non-existent VRF, the VRF is created and the route
  is added to it.
- When deleting routes for a VRF, all routes inside the VRF are deleted, but the VRF
  is not deleted.
options:
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the NX-OS device
      by executing the following commands in order B(show running-config | include
      '^ip(v6)* route') and B(show running-config | section '^vrf context').
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  config:
    description:
    - A list of configurations for static routes
    type: list
    elements: dict
    suboptions:
      vrf:
        description:
        - The VRF to which the static route(s) belong
        type: str
      address_families:
        description: A dictionary specifying the address family to which the static
          route(s) belong.
        type: list
        elements: dict
        suboptions:
          afi:
            description:
            - Specifies the top level address family indicator.
            type: str
            choices: [ipv4, ipv6]
            required: true
          routes:
            description: A dictionary that specifies the static route configurations
            elements: dict
            type: list
            suboptions:
              dest:
                description:
                - Destination prefix of static route
                - The address format is <ipv4/v6 address>/<mask>
                - The mask is number in range 0-32 for IPv4 and in range 0-128 for
                  IPv6
                type: str
                required: true
              next_hops:
                description:
                - Details of route to be taken
                type: list
                elements: dict
                suboptions:
                  forward_router_address:
                    description:
                    - IP address of the next hop router
                    type: str
                    # required: True
                  interface:
                    description:
                    - Outgoing interface to take. For anything except 'Null0', then
                      next hop IP address should also be configured.
                    type: str
                  admin_distance:
                    description:
                    - Preference or administrative distance of route (range 1-255)
                    type: int
                  route_name:
                    description:
                    - Name of the static route
                    type: str
                  tag:
                    description:
                    - Route tag value (numeric)
                    type: int
                  track:
                    description:
                    - Track value (range 1 - 512). Track must already be configured
                      on the device before adding the route.
                    type: int
                  dest_vrf:
                    description:
                    - VRF of the destination
                    type: str
  state:
    description:
      - The state the configuration should be left in
      - The states I(rendered), I(gathered) and I(parsed) does not perform any change
        on the device.
      - The state I(rendered) will transform the configuration in C(config) option to
        platform specific CLI commands which will be returned in the I(rendered) key
        within the result. For state I(rendered) active connection to remote host is
        not required.
      - The state C(deleted) does not support or guarantee granular deletion of configuration
        the playbook should act as source of truth, and the desired state of the resouce is what
        the playbook should reflect. Use C(overridden) or C(replaced) to get extra configuration
        removed.
      - The state I(gathered) will fetch the running configuration from device and transform
        it into structured data in the format as per the resource module argspec and
        the value is returned in the I(gathered) key within the result.
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into JSON format as per the resource module parameters and the
        value is returned in the I(parsed) key within the result. The value of C(running_config)
        option should be the same format as the output of command
        I(show running-config | section '^ip(v6)* route') executed on device. For state I(parsed) active
        connection to remote host is not required.
    type: str
    choices:
    - deleted
    - merged
    - overridden
    - replaced
    - gathered
    - rendered
    - parsed
    default: merged
"""

EXAMPLES = """
# Using deleted - delete all

# Before state:
# -------------
#
# switch# show running-config | include '^ip(v6)* route'
# ip route 192.0.2.32/28 192.0.2.12 name new_route
# ip route 192.0.2.26/24 192.0.2.13 tag 12
# switch# show running-config | section '^vrf context'

- name: Delete all routes
  cisco.nxos.nxos_static_routes:
    state: deleted

# Task Output
# -----------
#
# before:
#   - address_families:
#       - afi: ipv4
#         routes:
#           - dest: 192.0.2.0/24
#             next_hops:
#               - forward_router_address: 192.0.2.13
#                 tag: 12
#           - dest: 192.0.2.32/28
#             next_hops:
#               - forward_router_address: 192.0.2.12
#                 route_name: new_route
# commands:
# - no ip route 192.0.2.0/24 192.0.2.13 tag 12
# - no ip route 192.0.2.32/28 192.0.2.12 name new_route
# after: []

# After state:
# ------------
# switch# show running-config | include '^ip(v6)* route'
# switch# show running-config | section '^vrf context'

# Using deleted - vrf based

# Before state:
# ------------
#
# switch# show running-config | include '^ip(v6)* route'
# ip route 192.0.2.16/28 192.0.2.24 name new_route
# ip route 192.0.2.80/28 192.0.2.26 tag 12
# switch# show running-config | section '^vrf context'
# vrf context trial_vrf
# ip route 192.0.2.64/28 192.0.2.22 tag 4
# ip route 192.0.2.64/28 192.0.2.23 name merged_route 1
# ipv6 route 2200:10::/36 2048:ae12::1 vrf dest 5

- name: Delete routes based on VRF
  cisco.nxos.nxos_static_routes:
    config:
      - vrf: trial_vrf
    state: deleted

# Task Output
# -----------
#
# before:
#   - address_families:
#       - afi: ipv4
#         routes:
#           - dest: 192.0.2.64/28
#             next_hops:
#               - forward_router_address: 192.0.2.22
#                 tag: 4
#               - admin_distance: 1
#                 forward_router_address: 192.0.2.23
#                 route_name: merged_route
#       - afi: ipv6
#         routes:
#           - dest: '2200:10::/36'
#             next_hops:
#               - admin_distance: 5
#                 dest_vrf: dest
#                 forward_router_address: '2048:ae12::1'
#     vrf: trial_vrf
#   - address_families:
#       - afi: ipv4
#         routes:
#           - dest: 192.0.2.16/28
#             next_hops:
#               - forward_router_address: 192.0.2.24
#                 route_name: new_route
#           - dest: 192.0.2.80/28
#             next_hops:
#               - forward_router_address: 192.0.2.26
#                 tag: 12
# commands:
# - vrf context trial_vrf
# - no ip route 192.0.2.64/28 192.0.2.22 tag 4
# - no ip route 192.0.2.64/28 192.0.2.23 name merged_route 1
# - no ipv6 route 2200:10::/36 2048:ae12::1 vrf dest 5
# after:
#   - address_families:
#       - afi: ipv4
#         routes:
#           - dest: 192.0.2.16/28
#             next_hops:
#               - forward_router_address: 192.0.2.24
#                 route_name: new_route
#           - dest: 192.0.2.80/28
#             next_hops:
#               - forward_router_address: 192.0.2.26
#                 tag: 12

# After state:
# -----------
#
# switch# show running-config | include '^ip(v6)* route'
# ip route 192.0.2.16/28 192.0.2.24 name new_route
# ip route 192.0.2.80/28 192.0.2.26 tag 12
# switch# show running-config | section '^vrf context'
# vrf context trial_vrf

# Using deleted - afi based

# Before state:
# ------------
#
# switch# show running-config | include '^ip(v6)* route'
# ip route 192.0.2.16/28 192.0.2.24 name new_route
# ip route 192.0.2.80/28 192.0.2.26 tag 12
# switch# show running-config | section '^vrf context'
# vrf context trial_vrf
# ip route 192.0.2.64/28 192.0.2.22 tag 4
# ip route 192.0.2.64/28 192.0.2.23 name merged_route 1
# ipv6 route 2200:10::/36 2048:ae12::1 vrf dest 5

- name: Delete routes based on AFI in a VRF
  cisco.nxos.nxos_static_routes:
    config:
      - vrf: trial_vrf
        address_families:
          - afi: ipv4
    state: deleted

# Task Output
# -----------
#
# before:
#   - address_families:
#       - afi: ipv4
#         routes:
#           - dest: 192.0.2.64/28
#             next_hops:
#               - forward_router_address: 192.0.2.22
#                 tag: 4
#               - admin_distance: 1
#                 forward_router_address: 192.0.2.23
#                 route_name: merged_route
#       - afi: ipv6
#         routes:
#           - dest: '2200:10::/36'
#             next_hops:
#               - admin_distance: 5
#                 dest_vrf: dest
#                 forward_router_address: '2048:ae12::1'
#     vrf: trial_vrf
#   - address_families:
#       - afi: ipv4
#         routes:
#           - dest: 192.0.2.16/28
#             next_hops:
#               - forward_router_address: 192.0.2.24
#                 route_name: new_route
#           - dest: 192.0.2.80/28
#             next_hops:
#               - forward_router_address: 192.0.2.26
#                 tag: 12
# commands:
# - vrf context trial_vrf
# - no ip route 192.0.2.64/28 192.0.2.22 tag 4
# - no ip route 192.0.2.64/28 192.0.2.23 name merged_route 1
# after:
#   - address_families:
#       - afi: ipv6
#         routes:
#           - dest: '2200:10::/36'
#             next_hops:
#               - admin_distance: 5
#                 dest_vrf: dest
#                 forward_router_address: '2048:ae12::1'
#     vrf: trial_vrf
#   - address_families:
#       - afi: ipv4
#         routes:
#           - dest: 192.0.2.16/28
#             next_hops:
#               - forward_router_address: 192.0.2.24
#                 route_name: new_route
#           - dest: 192.0.2.80/28
#             next_hops:
#               - forward_router_address: 192.0.2.26
#                 tag: 12

# After state:
# -----------
#
# switch# show running-config | include '^ip(v6)* route'
# ip route 192.0.2.16/28 192.0.2.24 name new_route
# ip route 192.0.2.80/28 192.0.2.26 tag 12
# switch# show running-config | section '^vrf context'
# vrf context trial_vrf
# ipv6 route 2200:10::/36 2048:ae12::1 vrf dest 5

# Using merged

# Before state:
# -------------
# switch# show running-config | include '^ip(v6)* route'
# switch# show running-config | section '^vrf context'

- name: Merge new static route configuration
  cisco.nxos.nxos_static_routes:
    config:
      - vrf: trial_vrf
        address_families:
          - afi: ipv4
            routes:
              - dest: 192.0.2.64/24
                next_hops:
                  - forward_router_address: 192.0.2.22
                    tag: 4
                    admin_distance: 2
      - address_families:
          - afi: ipv4
            routes:
              - dest: 192.0.2.16/24
                next_hops:
                  - forward_router_address: 192.0.2.24
                    route_name: new_route
          - afi: ipv6
            routes:
              - dest: '2001:db8::/64'
                next_hops:
                  - interface: eth1/3
                    forward_router_address: '2001:db8::12'
    state: merged

# Task Output
# -----------
#
# before:[]
# commands:
# - vrf context trial_vrf
# - ip route 192.0.2.64/24 192.0.2.22 tag 4 2
# - ip route 192.0.2.16/24 192.0.2.24 name new_route
# - ipv6 route 2001:db8::/64 Ethernet1/3 2001:db8::12
# after:
#     - vrf: trial_vrf
#       address_families:
#       - afi: ipv4
#         routes:
#         - dest: 192.0.2.64/24
#           next_hops:
#           - forward_router_address: 192.0.2.22
#             tag: 4
#             admin_distance: 2
#     - address_families:
#       - afi: ipv4
#         routes:
#         - dest: 192.0.2.16/24
#           next_hops:
#           - forward_router_address: 192.0.2.24
#             route_name: new_route
#       - afi: ipv6
#         routes:
#         - dest: 2001:db8::/64
#           next_hops:
#           - interface: eth1/3
#             forward_router_address: 2

# After state:
# ------------
#
# switch# show running-config | include '^ip(v6)* route'
# ip route 192.0.2.16/24 192.0.2.24 name new_route
# ipv6 route 2001:db8::/64 Ethernet1/3 2001:db8::12
# switch# show running-config | section '^vrf context'
# vrf context trial_vrf
#   ip route 192.0.2.0/24 192.0.2.22 tag 4 2

# Using overridden

# Before state:
# -------------
#
# switch# show running-config | include '^ip(v6)* route'
# ip route 192.0.2.16/28 192.0.2.24 name new_route
# ip route 192.0.2.80/28 192.0.2.26 tag 12
# switch# show running-config | section '^vrf context'
# vrf context trial_vrf
# ip route 192.0.2.64/28 192.0.2.22 tag 4
# ip route 192.0.2.64/28 192.0.2.23 name merged_route 1

- name: Overridden existing static route configuration with new configuration
  cisco.nxos.nxos_static_routes:
    config:
      - vrf: trial_vrf
        address_families:
          - afi: ipv4
            routes:
              - dest: 192.0.2.16/28
                next_hops:
                  - forward_router_address: 192.0.2.23
                    route_name: overridden_route1
                    admin_distance: 3
                  - forward_router_address: 192.0.2.45
                    route_name: overridden_route2
                    dest_vrf: destinationVRF
                    interface: Ethernet1/2
    state: overridden

# Task Output
# -----------
#
# before:
#   - address_families:
#       - afi: ipv4
#         routes:
#           - dest: 192.0.2.64/28
#             next_hops:
#               - forward_router_address: 192.0.2.22
#                 tag: 4
#               - admin_distance: 1
#                 forward_router_address: 192.0.2.23
#                 route_name: merged_route
#     vrf: trial_vrf
#   - address_families:
#       - afi: ipv4
#         routes:
#           - dest: 192.0.2.16/28
#             next_hops:
#               - forward_router_address: 192.0.2.24
#                 route_name: new_route
#           - dest: 192.0.2.80/28
#             next_hops:
#               - forward_router_address: 192.0.2.26
#                 tag: 12
# commands:
# - no ip route 192.0.2.16/28 192.0.2.24 name new_route
# - no ip route 192.0.2.80/28 192.0.2.26 tag 12
# - vrf context trial_vrf
# - no ip route 192.0.2.64/28 192.0.2.22 tag 4
# - no ip route 192.0.2.64/28 192.0.2.23 name merged_route 1
# - ip route 192.0.2.16/28 192.0.2.23 name overridden_route1 3
# - ip route 192.0.2.16/28 Ethernet1/2 192.0.2.45 vrf destinationVRF name overridden_route2
# after:
#   - address_families:
#       - afi: ipv4
#         routes:
#           - dest: 192.0.2.16/28
#             next_hops:
#               - admin_distance: 3
#                 forward_router_address: 192.0.2.23
#                 route_name: overridden_route1
#               - dest_vrf: destinationVRF
#                 forward_router_address: 192.0.2.45
#                 interface: Ethernet1/2
#                 route_name: overridden_route2
#    vrf: trial_vrf

# After state:
# ------------
#
# switch# show running-config | include '^ip(v6)* route'
# switch# show running-config | section '^vrf context'
# vrf context trial_vrf
#   ip route 192.0.2.16/28 192.0.2.23 name overridden_route1 3
#   ip route 192.0.2.16/28 Ethernet1/2 192.0.2.45 vrf destinationVRF name overridden_route2

# Using replaced

# Before state:
# ------------
#
# switch# show running-config | include '^ip(v6)* route'
# ip route 192.0.2.16/28 192.0.2.24 name new_route
# ip route 192.0.2.80/28 192.0.2.26 tag 12
# switch# show running-config | section '^vrf context'
# vrf context trial_vrf
# ip route 192.0.2.64/28 192.0.2.22 tag 4
# ip route 192.0.2.64/28 192.0.2.23 name merged_route 1

- name: Replaced the existing static configuration of a prefix with new configuration
  cisco.nxos.nxos_static_routes:
    config:
      - address_families:
          - afi: ipv4
            routes:
              - dest: 192.0.2.16/28
                next_hops:
                  - forward_router_address: 192.0.2.23
                    route_name: replaced_route1
                    admin_distance: 3
                  - forward_router_address: 192.0.2.45
                    route_name: replaced_route2
                    dest_vrf: destinationVRF
                    interface: Ethernet1/2
    state: replaced

# Task Output
# -----------
#
# before:
#   - address_families:
#       - afi: ipv4
#         routes:
#           - dest: 192.0.2.64/28
#             next_hops:
#               - forward_router_address: 192.0.2.22
#                 tag: 4
#               - admin_distance: 1
#                 forward_router_address: 192.0.2.23
#                 route_name: merged_route
#     vrf: trial_vrf
#   - address_families:
#       - afi: ipv4
#         routes:
#           - dest: 192.0.2.16/28
#             next_hops:
#               - forward_router_address: 192.0.2.24
#                 route_name: new_route
#           - dest: 192.0.2.80/28
#             next_hops:
#               - forward_router_address: 192.0.2.26
#                 tag: 12
# commands:
# - no ip route 192.0.2.16/28 192.0.2.24 name new_route
# - ip route 192.0.2.16/28 192.0.2.23 name replaced_route1 3
# - ip route 192.0.2.16/28 Ethernet1/2 192.0.2.45 vrf destinationVRF name replaced_route2
# after:
#   - address_families:
#       - afi: ipv4
#         routes:
#           - dest: 192.0.2.64/28
#             next_hops:
#               - forward_router_address: 192.0.2.22
#                 tag: 4
#               - admin_distance: 1
#                 forward_router_address: 192.0.2.23
#                 route_name: merged_route
#     vrf: trial_vrf
#   - address_families:
#       - afi: ipv4
#         routes:
#           - dest: 192.0.2.16/28
#             next_hops:
#               - admin_distance: 3
#                 forward_router_address: 192.0.2.23
#                 route_name: replaced_route1
#               - dest_vrf: destinationVRF
#                 forward_router_address: 192.0.2.45
#                 interface: Ethernet1/2
#                 route_name: replaced_route2
#           - dest: 192.0.2.80/28
#             next_hops:
#               - forward_router_address: 192.0.2.26
#                 tag: 12

# After state:
# ------------
#
# switch# show running-config | include '^ip(v6)* route'
# ip route 192.0.2.16/28 192.0.2.23 name replaced_route1 3
# ip route 192.0.2.16/28 Ethernet1/2 192.0.2.45 vrf destinationVRF name replaced_route2
# ip route 192.0.2.80/28 192.0.2.26 tag 12
# switch# show running-config | section '^vrf context'
# vrf context trial_vrf
# ip route 192.0.2.64/28 192.0.2.22 tag 4
# ip route 192.0.2.64/28 192.0.2.23 name merged_route 1


# Using gathered

# Before state:
# -------------
#
# switch# show running-config | include '^ip(v6)* route'
# ipv6 route 2001:db8:12::/32  2001:db8::12
# switch# show running-config | section '^vrf context'
# vrf context Test
#    ip route 192.0.2.48/28 192.0.2.13
#    ip route 192.0.2.48/28 192.0.2.14 5

- name: Gather the existing configuration
  cisco.nxos.nxos_static_routes:
    state: gathered

# Task Output
# -----------
#
# gathered:
#     - vrf: Test
#       address_families:
#         - afi: ipv4
#           routes:
#             - dest: 192.0.2.48/28
#               next_hops:
#                 - forward_router_address: 192.0.2.13
#
#                 - forward_router_address: 192.0.2.14
#                   admin_distance: 5
#
#     - address_families:
#         - afi: ipv6
#           routes:
#             - dest: 2001:db8:12::/32
#               next_hops:
#                 - forward_router_address: 2001:db8::12


# Using rendered

- name: Render required configuration to be pushed to the device
  cisco.nxos.nxos_static_routes:
    config:
      - address_families:
          - afi: ipv4
            routes:
              - dest: 192.0.2.48/28
                next_hops:
                  - forward_router_address: 192.0.2.13
          - afi: ipv6
            routes:
              - dest: 2001:db8::/64
                next_hops:
                  - interface: eth1/3
                    forward_router_address: 2001:db8::12
    state: rendered

# Task Output
# -----------
#
# rendered:
#   vrf context default
#   ip route 192.0.2.48/28 192.0.2.13
#   ipv6 route 2001:db8::/64 Ethernet1/3 2001:db8::12

# Using parsed

- name: Parse the config to structured data
  cisco.nxos.nxos_static_routes:
    state: parsed
    running_config: |
      ipv6 route 2002:db8:12::/32 2002:db8:12::1
      vrf context Test
        ip route 192.0.2.48/28 192.0.2.13
        ip route 192.0.2.48/28 192.0.2.14 5

# Task Output
# -----------
#
# parsed:
#     - vrf: Test
#       address_families:
#         - afi: ipv4
#           routes:
#             - dest: 192.0.2.48/28
#               next_hops:
#                 - forward_router_address: 192.0.2.13
#                 - forward_router_address: 192.0.2.14
#                   admin_distance: 5
#     - address_families:
#         - afi: ipv6
#           routes:
#             - dest: 2002:db8:12::/32
#               next_hops:
#                 - forward_router_address: 2002:db8:12::1
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample:
    - ip route 192.0.2.16/28 192.0.2.24 name new_route
    - vrf context trial_vrf
    - ip route 192.0.2.16/28 192.0.2.23 name overridden_route1 3
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - ip route 192.0.2.16/28 192.0.2.24 name new_route
    - vrf context trial_vrf
    - ip route 192.0.2.16/28 192.0.2.23 name overridden_route1 3
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""


from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.static_routes.static_routes import (
    Static_routesArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.config.static_routes.static_routes import (
    Static_routes,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """

    module = AnsibleModule(
        argument_spec=Static_routesArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Static_routes(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# Copyright (c) 2019, Guillaume Martinez (lunik@tiwabbit.fr)
# Copyright (c) 2018, Marcus Watkins <marwatk@marcuswatkins.net>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import traceback
import typing as t
from urllib.parse import urljoin

from ansible.module_utils.basic import missing_required_lib

from ansible_collections.community.general.plugins.module_utils._version import LooseVersion

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule


def _determine_list_all_kwargs(version: str) -> dict[str, t.Any]:
    gitlab_version = LooseVersion(version)
    if gitlab_version >= LooseVersion("4.0.0"):
        # 4.0.0 removed 'as_list'
        return {"iterator": True, "per_page": 100}
    elif gitlab_version >= LooseVersion("3.7.0"):
        # 3.7.0 added 'get_all'
        return {"as_list": False, "get_all": True, "per_page": 100}
    else:
        return {"as_list": False, "all": True, "per_page": 100}


GITLAB_IMP_ERR: str | None = None
try:
    import gitlab
    import requests

    HAS_GITLAB_PACKAGE = True
    list_all_kwargs = _determine_list_all_kwargs(gitlab.__version__)
except Exception:
    gitlab = None  # type: ignore
    GITLAB_IMP_ERR = traceback.format_exc()
    HAS_GITLAB_PACKAGE = False
    list_all_kwargs = {}


def auth_argument_spec(spec: dict[str, t.Any] | None = None) -> dict[str, t.Any]:
    arg_spec = dict(
        ca_path=dict(type="str"),
        api_token=dict(type="str", no_log=True),
        api_oauth_token=dict(type="str", no_log=True),
        api_job_token=dict(type="str", no_log=True),
    )
    if spec:
        arg_spec.update(spec)
    return arg_spec


def find_project(gitlab_instance, identifier):
    try:
        project = gitlab_instance.projects.get(identifier)
    except Exception:
        current_user = gitlab_instance.user
        try:
            project = gitlab_instance.projects.get(f"{current_user.username}/{identifier}")
        except Exception:
            return None

    return project


def find_group(gitlab_instance, identifier):
    try:
        group = gitlab_instance.groups.get(identifier)
    except Exception:
        return None

    return group


def ensure_gitlab_package(module: AnsibleModule, min_version=None) -> None:
    if not HAS_GITLAB_PACKAGE:
        module.fail_json(
            msg=missing_required_lib("python-gitlab", url="https://python-gitlab.readthedocs.io/en/stable/"),
            exception=GITLAB_IMP_ERR,
        )
    gitlab_version = gitlab.__version__
    if min_version is not None and LooseVersion(gitlab_version) < LooseVersion(min_version):
        module.fail_json(
            msg=(
                f"This module requires python-gitlab Python module >= {min_version} (installed version: "
                f"{gitlab_version}). Please upgrade python-gitlab to version {min_version} or above."
            )
        )


def gitlab_authentication(module: AnsibleModule, min_version=None) -> gitlab.Gitlab:
    ensure_gitlab_package(module, min_version=min_version)

    gitlab_url = module.params["api_url"]
    validate_certs = module.params["validate_certs"]
    ca_path = module.params["ca_path"]
    gitlab_user = module.params["api_username"]
    gitlab_password = module.params["api_password"]
    gitlab_token = module.params["api_token"]
    gitlab_oauth_token = module.params["api_oauth_token"]
    gitlab_job_token = module.params["api_job_token"]

    verify = ca_path if validate_certs and ca_path else validate_certs

    try:
        # We can create an oauth_token using a username and password
        # https://docs.gitlab.com/ee/api/oauth2.html#authorization-code-flow
        if gitlab_user:
            data = {"grant_type": "password", "username": gitlab_user, "password": gitlab_password}
            resp = requests.post(urljoin(gitlab_url, "oauth/token"), data=data, verify=verify)
            resp_data = resp.json()
            gitlab_oauth_token = resp_data["access_token"]

        gitlab_instance = gitlab.Gitlab(
            url=gitlab_url,
            ssl_verify=verify,
            private_token=gitlab_token,
            oauth_token=gitlab_oauth_token,
            job_token=gitlab_job_token,
            api_version="4",
        )
        gitlab_instance.auth()
    except (gitlab.exceptions.GitlabAuthenticationError, gitlab.exceptions.GitlabGetError) as e:
        module.fail_json(msg=f"Failed to connect to GitLab server: {e}")
    except gitlab.exceptions.GitlabHttpError as e:
        module.fail_json(
            msg=(
                f"Failed to connect to GitLab server: {e}. GitLab remove Session API now "
                "that private tokens are removed from user API endpoints since version 10.2."
            )
        )

    return gitlab_instance


def filter_returned_variables(gitlab_variables) -> list[dict[str, t.Any]]:
    # pop properties we don't know
    existing_variables = [dict(x.attributes) for x in gitlab_variables]
    KNOWN = [
        "key",
        "value",
        "description",
        "masked",
        "hidden",
        "protected",
        "variable_type",
        "environment_scope",
        "raw",
    ]
    for item in existing_variables:
        for key in list(item.keys()):
            if key not in KNOWN:
                item.pop(key)
    return existing_variables


def vars_to_variables(
    vars: dict[str, str | int | float | dict[str, t.Any]], module: AnsibleModule
) -> list[dict[str, t.Any]]:
    # transform old vars to new variables structure
    variables = []
    for item, value in vars.items():
        if isinstance(value, (str, int, float)):
            variables.append(
                {
                    "name": item,
                    "value": str(value),
                    "description": None,
                    "masked": False,
                    "protected": False,
                    "hidden": False,
                    "raw": False,
                    "variable_type": "env_var",
                }
            )

        elif isinstance(value, dict):
            new_item = {
                "name": item,
                "value": value.get("value"),
                "description": value.get("description"),
                "masked": value.get("masked"),
                "hidden": value.get("hidden"),
                "protected": value.get("protected"),
                "raw": value.get("raw"),
                "variable_type": value.get("variable_type"),
            }

            if value.get("environment_scope"):
                new_item["environment_scope"] = value.get("environment_scope")

            variables.append(new_item)

        else:
            module.fail_json(msg="value must be of type string, integer, float or dict")

    return variables



# Copyright (c) 2017-18, Ansible Project
# Copyright (c) 2017-18, Abhijeet Kasurde (akasurde@redhat.com)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:
    # Parameters for FreeIPA/IPA modules
    DOCUMENTATION = r"""
options:
  ipa_port:
    description:
      - Port of FreeIPA / IPA server.
      - If the value is not specified in the task, the value of environment variable E(IPA_PORT) is used instead.
      - If both the environment variable E(IPA_PORT) and the value are not specified in the task, then default value is set.
    type: int
    default: 443
  ipa_host:
    description:
      - IP or hostname of IPA server.
      - If the value is not specified in the task, the value of environment variable E(IPA_HOST) is used instead.
      - If both the environment variable E(IPA_HOST) and the value are not specified in the task, then DNS is used to try
        to discover the FreeIPA server.
      - The relevant entry needed in FreeIPA is the C(ipa-ca) entry.
      - If neither the DNS entry, nor the environment E(IPA_HOST), nor the value are available in the task, then the default
        value is used.
    type: str
    default: ipa.example.com
  ipa_user:
    description:
      - Administrative account used on IPA server.
      - If the value is not specified in the task, the value of environment variable E(IPA_USER) is used instead.
      - If both the environment variable E(IPA_USER) and the value are not specified in the task, then default value is set.
    type: str
    default: admin
  ipa_pass:
    description:
      - Password of administrative user.
      - If the value is not specified in the task, the value of environment variable E(IPA_PASS) is used instead.
      - Note that if the C(urllib_gssapi) library is available, it is possible to use GSSAPI to authenticate to FreeIPA.
      - If the environment variable E(KRB5CCNAME) is available, the module uses this Kerberos credentials cache to authenticate
        to the FreeIPA server.
      - If the environment variable E(KRB5_CLIENT_KTNAME) is available, and E(KRB5CCNAME) is not; the module uses this Kerberos
        keytab to authenticate.
      - If GSSAPI is not available, the usage of O(ipa_pass) is required.
    type: str
  ipa_prot:
    description:
      - Protocol used by IPA server.
      - If the value is not specified in the task, the value of environment variable E(IPA_PROT) is used instead.
      - If both the environment variable E(IPA_PROT) and the value are not specified in the task, then default value is set.
    type: str
    choices: [http, https]
    default: https
  validate_certs:
    description:
      - This only applies if O(ipa_prot) is V(https).
      - If set to V(false), the SSL certificates are not validated.
      - This should only set to V(false) used on personally controlled sites using self-signed certificates.
    type: bool
    default: true
  ipa_timeout:
    description:
      - Specifies idle timeout (in seconds) for the connection.
      - For bulk operations, you may want to increase this in order to avoid timeout from IPA server.
      - If the value is not specified in the task, the value of environment variable E(IPA_TIMEOUT) is used instead.
      - If both the environment variable E(IPA_TIMEOUT) and the value are not specified in the task, then default value is
        set.
    type: int
    default: 10
"""

    CONNECTION_NOTES = r"""
options: {}
notes:
  - This module uses JSON-RPC over HTTP(S) to communicate with the FreeIPA server.
    If you need to enroll the managed node into FreeIPA realm, you might want to consider using the collection
    L(freeipa.ansible_freeipa, https://galaxy.ansible.com/ui/repo/published/freeipa/ansible_freeipa/), but shell access to one
    node from the realm is required to manage the deployment.
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_iscsi_adapter_policy
short_description: Manage iSCSI Adapter Policies for Cisco Intersight
description:
  - Create, update, and delete iSCSI Adapter Policies on Cisco Intersight.
  - iSCSI adapter policies configure timeout and retry settings for iSCSI adapters.
  - These policies control TCP connection timeout, DHCP timeout, and LUN busy retry behavior.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/vnic/IscsiAdapterPolicies/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Policies created within a Custom Organization are applicable only to devices in the same Organization.
      - Use 'default' for the default organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the iSCSI Adapter Policy.
      - Must be unique within the organization.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the iSCSI Adapter Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  connection_time_out:
    description:
      - The number of seconds to wait until Cisco UCS assumes that the initial login has failed and the iSCSI adapter is unavailable.
      - TCP connection timeout value.
      - Valid range is 0 to 255 seconds.
    type: int
    default: 15
  dhcp_timeout:
    description:
      - The number of seconds to wait before the initiator assumes that the DHCP server is unavailable.
      - Valid range is 60 to 300 seconds.
    type: int
    default: 60
  lun_busy_retry_count:
    description:
      - The number of times to retry the connection in case of a failure during iSCSI LUN discovery.
      - Valid range is 0 to 60 retries.
    type: int
    default: 15
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create iSCSI Adapter Policy with default settings
  cisco.intersight.intersight_iscsi_adapter_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "iscsi-adapter-default"
    description: "iSCSI adapter policy with default timeout values"
    state: present

- name: Create iSCSI Adapter Policy with custom timeout settings
  cisco.intersight.intersight_iscsi_adapter_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "iscsi-adapter-custom"
    description: "iSCSI adapter policy with custom timeouts"
    connection_time_out: 30
    dhcp_timeout: 120
    lun_busy_retry_count: 30
    tags:
      - Key: Environment
        Value: Production
    state: present

- name: Create iSCSI Adapter Policy with extended timeouts
  cisco.intersight.intersight_iscsi_adapter_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "Engineering"
    name: "iscsi-adapter-extended"
    description: "iSCSI adapter policy with extended timeout values"
    connection_time_out: 60
    dhcp_timeout: 300
    lun_busy_retry_count: 60
    state: present

- name: Create iSCSI Adapter Policy with minimal timeouts
  cisco.intersight.intersight_iscsi_adapter_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "iscsi-adapter-minimal"
    description: "iSCSI adapter policy with minimal timeout values"
    connection_time_out: 0
    dhcp_timeout: 60
    lun_busy_retry_count: 0
    state: present

- name: Update iSCSI Adapter Policy description
  cisco.intersight.intersight_iscsi_adapter_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "iscsi-adapter-default"
    description: "Updated iSCSI adapter policy description"
    state: present

- name: Delete iSCSI Adapter Policy
  cisco.intersight.intersight_iscsi_adapter_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "iscsi-adapter-default"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "iscsi-adapter-custom",
        "ObjectType": "vnic.IscsiAdapterPolicy",
        "ConnectionTimeOut": 30,
        "DhcpTimeout": 120,
        "LunBusyRetryCount": 30,
        "Moid": "1234567890abcdef12345678",
        "Description": "iSCSI adapter policy with custom timeouts",
        "Organization": {
            "Moid": "abcdef1234567890abcdef12",
            "ObjectType": "organization.Organization"
        },
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def validate_parameters(module):
    """
    Validate module parameters for iSCSI adapter policy configuration.
    """
    if module.params['state'] != 'present':
        return
    # Validate connection_time_out range (0-255)
    connection_time_out = module.params.get('connection_time_out')
    if connection_time_out is not None and (connection_time_out < 0 or connection_time_out > 255):
        module.fail_json(msg="Parameter 'connection_time_out' must be between 0 and 255 seconds")
    # Validate dhcp_timeout range (60-300)
    dhcp_timeout = module.params.get('dhcp_timeout')
    if dhcp_timeout is not None and (dhcp_timeout < 60 or dhcp_timeout > 300):
        module.fail_json(msg="Parameter 'dhcp_timeout' must be between 60 and 300 seconds")
    # Validate lun_busy_retry_count range (0-60)
    lun_busy_retry_count = module.params.get('lun_busy_retry_count')
    if lun_busy_retry_count is not None and (lun_busy_retry_count < 0 or lun_busy_retry_count > 60):
        module.fail_json(msg="Parameter 'lun_busy_retry_count' must be between 0 and 60 retries")


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        connection_time_out=dict(type='int', default=15),
        dhcp_timeout=dict(type='int', default=60),
        lun_busy_retry_count=dict(type='int', default=15),
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )
    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''
    # Validate module parameters
    validate_parameters(module)
    # Resource path used to configure policy
    resource_path = '/vnic/IscsiAdapterPolicies'
    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': module.params['organization'],
        },
        'Name': module.params['name'],
    }
    if module.params['state'] == 'present':
        intersight.api_body['ConnectionTimeOut'] = module.params['connection_time_out']
        intersight.api_body['DhcpTimeout'] = module.params['dhcp_timeout']
        intersight.api_body['LunBusyRetryCount'] = module.params['lun_busy_retry_count']
        intersight.set_tags_and_description()
    intersight.configure_policy_or_profile(resource_path=resource_path)
    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
filter plugin file for ipaddr filters: ipv6form
"""
from __future__ import absolute_import, division, print_function

from functools import partial

from ansible.errors import AnsibleFilterError

from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)
from ansible_collections.ansible.utils.plugins.plugin_utils.base.ipaddr_utils import _need_netaddr
from ansible_collections.ansible.utils.plugins.plugin_utils.base.ipaddress_utils import (
    _need_ipaddress,
    ip_address,
)


__metaclass__ = type


try:
    from jinja2.filters import pass_environment
except ImportError:
    from jinja2.filters import environmentfilter as pass_environment


try:
    import netaddr

    HAS_NETADDR = True
except ImportError:
    # in this case, we'll make the filters return error messages (see bottom)
    HAS_NETADDR = False
else:

    class mac_linux(netaddr.mac_unix):
        pass

    mac_linux.word_fmt = "%.2x"


DOCUMENTATION = """
    name: ipv6form
    author: Ashwini Mhatre (@amhatre)
    version_added: "2.11.0"
    short_description: This filter is designed to convert ipv6 address in different formats. For example expand, compressetc.
    description:
        - This filter is designed to convert ipv6 addresses in different formats.
    options:
        value:
            description:
            - individual ipv6 address input for ipv6_format plugin.
            type: str
            required: True
        format:
            type: str
            choices:
                ['compress', 'expand', 'x509']
            description: Different formats example. compress, expand, x509
"""

EXAMPLES = r"""
#### examples
# Ipv6form filter plugin with different format.
- name: Expand given Ipv6 address
  debug:
      msg: "{{ '1234:4321:abcd:dcba::17' | ansible.utils.ipv6form('expand') }}"

- name: Compress  given Ipv6 address
  debug:
      msg: "{{ '1234:4321:abcd:dcba:0000:0000:0000:0017' | ansible.utils.ipv6form('compress') }}"

- name: Covert given Ipv6 address in x509
  debug:
      msg: "{{ '1234:4321:abcd:dcba::17' | ansible.utils.ipv6form('x509') }}"

# TASK [Expand given Ipv6 address] ************************************************************************************
# task path: /home/amhatre/dev/playbook/test_ipform.yaml:7
# Loading collection ansible.utils from /home/amhatre/dev/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "1234:4321:abcd:dcba:0000:0000:0000:0017"
# }

# TASK [Compress  given Ipv6 address] *********************************************************************************
# task path: /home/amhatre/dev/playbook/test_ipform.yaml:11
# Loading collection ansible.utils from /home/amhatre/dev/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "1234:4321:abcd:dcba::17"
# }

# TASK [Covert given Ipv6 address in x509] ****************************************************************************
# task path: /home/amhatre/dev/playbook/test_ipform.yaml:15
# Loading collection ansible.utils from /home/amhatre/dev/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "1234:4321:abcd:dcba:0:0:0:17"
# }

# PLAY RECAP **********************************************************************************************************
# localhost                  : ok=3    changed=0    unreachable=0    failed=0    skipped=0    rescued=0    ignored=0
"""

RETURN = """
  data:
    type: str
    description:
      - Returns result ipv6 address in expected format.
"""


@pass_environment
def _ipv6form(*args, **kwargs):
    """Convert the given data from json to xml."""
    keys = ["value", "format"]
    data = dict(zip(keys, args[1:]))
    data.update(kwargs)
    aav = AnsibleArgSpecValidator(data=data, schema=DOCUMENTATION, name="ipv6form")
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    return ipv6form(**updated_data)


@_need_ipaddress
def ipv6form(value, format):
    try:
        if format == "expand":
            return ip_address(value).exploded
        elif format == "compress":
            return ip_address(value).compressed
        elif format == "x509":
            return _handle_x509(value)
    except ValueError:
        msg = "You must pass a valid IP address; {0} is invalid".format(value)
        raise AnsibleFilterError(msg)

    if not isinstance(format, str):
        msg = ("You must pass valid format; " "{0} is not a valid format").format(
            format,
        )
        raise AnsibleFilterError(msg)


def _handle_x509(value):
    """Convert ipv6 address into x509 format"""
    ip = netaddr.IPAddress(value)
    ipv6_oct = []
    ipv6address = ip.bits().split(":")
    for i in ipv6address:
        x = hex(int(i, 2))
        ipv6_oct.append(x.replace("0x", ""))
    return str(":".join(ipv6_oct))


class FilterModule(object):
    """IP address and network manipulation filters"""

    filter_map = {
        # This filter is designed to do ipv6 conversion in required format
        "ipv6form": _ipv6form,
    }

    def filters(self):
        """ipv6form filter"""
        if HAS_NETADDR:
            return self.filter_map
        else:
            return dict((f, partial(_need_netaddr, f)) for f in self.filter_map)



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_online_endpoint_invoke
version_added: "3.18.0"
short_description: Invoke ML Online Endpoint resources
description:
    - Invoke ML Online Endpoint resources.
options:
    name:
        description:
            - Name of the Azure ML online endpoint.
        required: true
        type: str
    deployment_name:
        description:
            - Name of the Azure ML Online deployment to target.
        required: false
        type: str
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    ml_workspace:
        description:
            - Name of the Azure ML workspace.
        required: true
        type: str
    request_file:
        description:
            - Local path to the JSON file containing the request data.
        required: false
        type: str
    local:
        description:
            - Invoke all local endpoints.
        type: bool
        default: false

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Invoke ML Online Endpoint
  azure.azcollection.azure_rm_ml_online_endpoint_invoke:
    name: endpoint-command-ansible
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace
    request_file: ./targets/azure_rm_ml_online_deployment/files/sample-request.json
'''

RETURN = '''
changed:
    description:
        - Will return True if successfully invoked
    returned: always
    type: bool
    sample: true
ml_online_endpoint_invoke:
    description:
        - Online Endpoint Invoked that match the name and deployment.
    returned: always
    type: dict
    sample: [11055.977245525679, 4503.079536107787]
'''  # NOQA


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLOnlineEndpointInvoke(MLClientCommon):
    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=True,
            ),
            deployment_name=dict(
                type='str',
                required=False,
            ),
            resource_group=dict(
                type='str',
                required=True,
            ),
            ml_workspace=dict(
                type='str',
                required=True,
            ),
            request_file=dict(
                type='str',
                required=False,
            ),
            local=dict(
                type='bool',
                default=False,
            ),
        )

        self._client = None
        self.ml_registry = None

        self.results = dict(
            changed=False,
            ml_online_endpoint_invoke=None
        )

        super(AzureRMMLOnlineEndpointInvoke, self).__init__(self.module_arg_spec,
                                                            supports_tags=False,
                                                            supports_check_mode=False,
                                                            facts_module=True,
                                                            )

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        input_data_asset = None

        ml_online_endpoint_invoke = self.client.online_endpoints.invoke(
            endpoint_name=self.name,
            request_file=self.request_file,
            input_data=input_data_asset,
            deployment_name=self.deployment_name,
            local=self.local,
        )

        self.results['changed'] = True
        self.results['ml_online_endpoint_invoke'] = ml_online_endpoint_invoke
        return self.results


def main():
    AzureRMMLOnlineEndpointInvoke()


if __name__ == '__main__':
    main()



# Based on local.py (c) 2012, Michael DeHaan <michael.dehaan@gmail.com>
#
# (c) 2013, Maykel Moya <mmoya@speedyrails.com>
# (c) 2015, Toshio Kuratomi <tkuratomi@ansible.com>
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author: Maykel Moya (!UNKNOWN) <mmoya@speedyrails.com>
name: chroot
short_description: Interact with local chroot
description:
  - Run commands or put/fetch files to an existing chroot on the Ansible controller.
options:
  remote_addr:
    description:
      - The path of the chroot you want to access.
    type: string
    default: inventory_hostname
    vars:
      - name: inventory_hostname
      - name: ansible_host
  executable:
    description:
      - User specified executable shell.
    type: string
    ini:
      - section: defaults
        key: executable
    env:
      - name: ANSIBLE_EXECUTABLE
    vars:
      - name: ansible_executable
    default: /bin/sh
  chroot_exe:
    description:
      - User specified chroot binary.
    type: string
    ini:
      - section: chroot_connection
        key: exe
    env:
      - name: ANSIBLE_CHROOT_EXE
    vars:
      - name: ansible_chroot_exe
    default: chroot
  disable_root_check:
    description:
      - Do not check that the user is not root.
    ini:
      - section: chroot_connection
        key: disable_root_check
    env:
      - name: ANSIBLE_CHROOT_DISABLE_ROOT_CHECK
    vars:
      - name: ansible_chroot_disable_root_check
    default: false
    type: bool
    version_added: 7.3.0
"""

EXAMPLES = r"""
- hosts: chroots
  connection: community.general.chroot
  tasks:
    - debug:
        msg: "This is coming from chroot environment"
"""

import os
import os.path
import subprocess
import traceback
from shlex import quote as shlex_quote

from ansible.errors import AnsibleError
from ansible.module_utils.basic import is_executable
from ansible.module_utils.common.process import get_bin_path
from ansible.module_utils.common.text.converters import to_bytes
from ansible.plugins.connection import BUFSIZE, ConnectionBase
from ansible.utils.display import Display

display = Display()


class Connection(ConnectionBase):
    """Local chroot based connections"""

    transport = "community.general.chroot"
    has_pipelining = True
    # su currently has an undiagnosed issue with calculating the file
    # checksums (so copy, for instance, doesn't work right)
    # Have to look into that before re-enabling this
    has_tty = False

    default_user = "root"

    def __init__(self, play_context, new_stdin, *args, **kwargs):
        super().__init__(play_context, new_stdin, *args, **kwargs)

        self.chroot = self._play_context.remote_addr

        # do some trivial checks for ensuring 'host' is actually a chroot'able dir
        if not os.path.isdir(self.chroot):
            raise AnsibleError(f"{self.chroot} is not a directory")

        chrootsh = os.path.join(self.chroot, "bin/sh")
        # Want to check for a usable bourne shell inside the chroot.
        # is_executable() == True is sufficient.  For symlinks it
        # gets really complicated really fast.  So we punt on finding that
        # out.  As long as it is a symlink we assume that it will work
        if not (is_executable(chrootsh) or (os.path.lexists(chrootsh) and os.path.islink(chrootsh))):
            raise AnsibleError(f"{self.chroot} does not look like a chrootable dir (/bin/sh missing)")

    def _connect(self):
        """connect to the chroot"""
        if not self.get_option("disable_root_check") and os.geteuid() != 0:
            raise AnsibleError(
                "chroot connection requires running as root. "
                "You can override this check with the `disable_root_check` option."
            )

        if os.path.isabs(self.get_option("chroot_exe")):
            self.chroot_cmd = self.get_option("chroot_exe")
        else:
            try:
                self.chroot_cmd = get_bin_path(self.get_option("chroot_exe"))
            except ValueError as e:
                raise AnsibleError(str(e)) from e

        super()._connect()
        if not self._connected:
            display.vvv("THIS IS A LOCAL CHROOT DIR", host=self.chroot)
            self._connected = True

    def _buffered_exec_command(self, cmd, stdin=subprocess.PIPE):
        """run a command on the chroot.  This is only needed for implementing
        put_file() get_file() so that we don't have to read the whole file
        into memory.

        compared to exec_command() it looses some niceties like being able to
        return the process's exit code immediately.
        """
        executable = self.get_option("executable")
        local_cmd = [self.chroot_cmd, self.chroot, executable, "-c", cmd]

        display.vvv(f"EXEC {local_cmd}", host=self.chroot)
        local_cmd = [to_bytes(i, errors="surrogate_or_strict") for i in local_cmd]
        p = subprocess.Popen(local_cmd, shell=False, stdin=stdin, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

        return p

    def exec_command(self, cmd, in_data=None, sudoable=False):
        """run a command on the chroot"""
        super().exec_command(cmd, in_data=in_data, sudoable=sudoable)

        p = self._buffered_exec_command(cmd)

        stdout, stderr = p.communicate(in_data)
        return p.returncode, stdout, stderr

    @staticmethod
    def _prefix_login_path(remote_path):
        """Make sure that we put files into a standard path

        If a path is relative, then we need to choose where to put it.
        ssh chooses $HOME but we aren't guaranteed that a home dir will
        exist in any given chroot.  So for now we're choosing "/" instead.
        This also happens to be the former default.

        Can revisit using $HOME instead if it is a problem
        """
        if not remote_path.startswith(os.path.sep):
            remote_path = os.path.join(os.path.sep, remote_path)
        return os.path.normpath(remote_path)

    def put_file(self, in_path, out_path):
        """transfer a file from local to chroot"""
        super().put_file(in_path, out_path)
        display.vvv(f"PUT {in_path} TO {out_path}", host=self.chroot)

        out_path = shlex_quote(self._prefix_login_path(out_path))
        try:
            with open(to_bytes(in_path, errors="surrogate_or_strict"), "rb") as in_file:
                if not os.fstat(in_file.fileno()).st_size:
                    count = " count=0"
                else:
                    count = ""
                try:
                    p = self._buffered_exec_command(f"dd of={out_path} bs={BUFSIZE}{count}", stdin=in_file)
                except OSError as e:
                    raise AnsibleError("chroot connection requires dd command in the chroot") from e
                try:
                    stdout, stderr = p.communicate()
                except Exception as e:
                    traceback.print_exc()
                    raise AnsibleError(f"failed to transfer file {in_path} to {out_path}") from e
                if p.returncode != 0:
                    raise AnsibleError(f"failed to transfer file {in_path} to {out_path}:\n{stdout}\n{stderr}")
        except OSError as e:
            raise AnsibleError(f"file or module does not exist at: {in_path}") from e

    def fetch_file(self, in_path, out_path):
        """fetch a file from chroot to local"""
        super().fetch_file(in_path, out_path)
        display.vvv(f"FETCH {in_path} TO {out_path}", host=self.chroot)

        in_path = shlex_quote(self._prefix_login_path(in_path))
        try:
            p = self._buffered_exec_command(f"dd if={in_path} bs={BUFSIZE}")
        except OSError as e:
            raise AnsibleError("chroot connection requires dd command in the chroot") from e

        with open(to_bytes(out_path, errors="surrogate_or_strict"), "wb+") as out_file:
            try:
                chunk = p.stdout.read(BUFSIZE)
                while chunk:
                    out_file.write(chunk)
                    chunk = p.stdout.read(BUFSIZE)
            except Exception as e:
                traceback.print_exc()
                raise AnsibleError(f"failed to transfer file {in_path} to {out_path}") from e
            stdout, stderr = p.communicate()
            if p.returncode != 0:
                raise AnsibleError(f"failed to transfer file {in_path} to {out_path}:\n{stdout}\n{stderr}")

    def close(self):
        """terminate the connection; nothing to do here"""
        super().close()
        self._connected = False



#!/usr/bin/python

# Copyright (c) 2016, Peter Sagerson <psagers@ignorare.net>
# Copyright (c) 2016, Jiri Tyr <jiri.tyr@gmail.com>
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ldap_entry
short_description: Add or remove LDAP entries
description:
  - Add or remove LDAP entries. This module only asserts the existence or non-existence of an LDAP entry, not its attributes.
    To assert the attribute values of an entry, see M(community.general.ldap_attrs).
author:
  - Jiri Tyr (@jtyr)
requirements:
  - python-ldap
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  attributes:
    description:
      - If O(state=present), attributes necessary to create an entry. Existing entries are never modified. To assert specific
        attribute values on an existing entry, use M(community.general.ldap_attrs) module instead.
      - Each attribute value can be a string for single-valued attributes or a list of strings for multi-valued attributes.
      - If you specify values for this option in YAML, please note that you can improve readability for long string values
        by using YAML block modifiers as seen in the examples for this module.
      - Note that when using values that YAML/ansible-core interprets as other types, like V(yes), V(no) (booleans), or V(2.10)
        (float), make sure to quote them if these are meant to be strings. Otherwise the wrong values may be sent to LDAP.
      - If O(honor_binary=true), an attribute that includes the C(binary) option as per
        L(RFC 4522, https://www.rfc-editor.org/rfc/rfc4522.html#section-3) will be considered as binary. Its contents must be
        specified as Base64 and sent to the LDAP after decoding. If an attribute must be handled as binary without including
        the C(binary) option, it can be listed in O(binary_attributes).
        Support for binary values was added in community.general 12.5.0.
    type: dict
    default: {}
  binary_attributes:
    description:
      - If O(state=present), attributes whose values must be handled as raw sequences of bytes must be listed here.
      - The values provided for the attributes will be converted from Base64.
    type: list
    elements: str
    default: []
    version_added: 12.5.0
  honor_binary:
    description:
      - If O(state=present) and this option is V(true), attributes whose name include the V(binary) option
        will be treated as Base64-encoded byte sequences automatically, even if they are not listed in O(binary_attributes).
    type: bool
    default: false
    version_added: 12.5.0
  objectClass:
    description:
      - If O(state=present), value or list of values to use when creating the entry. It can either be a string or an actual
        list of strings.
    type: list
    elements: str
  state:
    description:
      - The target state of the entry.
    choices: [present, absent]
    default: present
    type: str
  recursive:
    description:
      - If O(state=delete), a flag indicating whether a single entry or the whole branch must be deleted.
    type: bool
    default: false
    version_added: 4.6.0
extends_documentation_fragment:
  - community.general._ldap.documentation
  - community.general._attributes
"""


EXAMPLES = r"""
- name: Make sure we have a parent entry for users
  community.general.ldap_entry:
    dn: ou=users,dc=example,dc=com
    objectClass: organizationalUnit

- name: Make sure we have an admin user
  community.general.ldap_entry:
    dn: cn=admin,dc=example,dc=com
    objectClass:
      - simpleSecurityObject
      - organizationalRole
      - myPhotoObject
    binary_attributes:
      - myPhoto
    attributes:
      description: An LDAP administrator
      userPassword: "{SSHA}tabyipcHzhwESzRaGA7oQ/SDoBZQOGND"
      myPhoto: >-
        /9j/4AAQSkZJRgABAQAAAQABAAD/4gIcSUNDX1BST0ZJTEUAAQEAAAIMbGNt
        cwIQAABtbnRyUkdCIFhZWiAH3AABABkAAwApADlhY3NwQVBQTAAAAAAAAAAA
        # ...

- name: Make sure a CA certificate is present
  community.general.ldap_attrs:
    dn: cn=ISRG Root X1,ou=ca,ou=certificates,dc=example,dc=org
    honor_binary: true
    objectClass:
      - pkiCA
      - applicationProcess
    attributes:
      cn: ISRG Root X1
      cACertificate;binary: >-
        MIIFazCCA1OgAwIBAgIRAIIQz7DSQONZRGPgu2OCiwAwDQYJKoZIhvcNAQELBQAw
        TzELMAkGA1UEBhMCVVMxKTAnBgNVBAoTIEludGVybmV0IFNlY3VyaXR5IFJlc2Vh
        # ...

- name: Set possible values for attributes elements
  community.general.ldap_entry:
    dn: cn=admin,dc=example,dc=com
    objectClass:
      - simpleSecurityObject
      - organizationalRole
    attributes:
      description: An LDAP Administrator
      roleOccupant:
        - cn=Chocs Puddington,ou=Information Technology,dc=example,dc=com
        - cn=Alice Stronginthebrain,ou=Information Technology,dc=example,dc=com
      olcAccess:
        - >-
          {0}to attrs=userPassword,shadowLastChange
          by self write
          by anonymous auth
          by dn="cn=admin,dc=example,dc=com" write
          by * none'
        - >-
          {1}to dn.base="dc=example,dc=com"
          by dn="cn=admin,dc=example,dc=com" write
          by * read

- name: Get rid of an old entry
  community.general.ldap_entry:
    dn: ou=stuff,dc=example,dc=com
    state: absent
    server_uri: ldap://localhost/
    bind_dn: cn=admin,dc=example,dc=com
    bind_pw: password

#
# The same as in the previous example but with the authentication details
# stored in the ldap_auth variable:
#
# ldap_auth:
#   server_uri: ldap://localhost/
#   bind_dn: cn=admin,dc=example,dc=com
#   bind_pw: password
#
# In the example below, 'args' is a task keyword, passed at the same level as the module
- name: Get rid of an old entry
  community.general.ldap_entry:
    dn: ou=stuff,dc=example,dc=com
    state: absent
  args: "{{ ldap_auth }}"
"""


RETURN = r"""
# Default return values
"""

import base64
import binascii
import traceback

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.common.text.converters import to_bytes

from ansible_collections.community.general.plugins.module_utils._ldap import (
    LdapGeneric,
    gen_specs,
    ldap_required_together,
)

LDAP_IMP_ERR = None
try:
    import ldap.controls
    import ldap.modlist

    HAS_LDAP = True
except ImportError:
    LDAP_IMP_ERR = traceback.format_exc()
    HAS_LDAP = False


class LdapEntry(LdapGeneric):
    def __init__(self, module):
        LdapGeneric.__init__(self, module)

        # Shortcuts
        self.state = self.module.params["state"]
        self.recursive = self.module.params["recursive"]
        self.binary = set(attr.lower() for attr in self.module.params["binary_attributes"])
        self.honor_binary = self.module.params["honor_binary"]

        # Add the objectClass into the list of attributes
        self.module.params["attributes"]["objectClass"] = self.module.params["objectClass"]

        # Load attributes
        if self.state == "present":
            self.attrs, bad_attrs = self._load_attrs()
            if bad_attrs:
                s_bad_attrs = ", ".join(bad_attrs)
                self.module.fail_json(msg=f"Invalid Base64-encoded attribute values for {s_bad_attrs}")

    def _is_binary(self, attr_name):
        """Check if an attribute must be considered binary."""
        lc_name = attr_name.lower()
        return (self.honor_binary and "binary" in lc_name.split(";")) or lc_name in self.binary

    def _load_attrs(self):
        """Turn attribute's value to array. Attribute values are converted to
        raw bytes, either by encoding the string itself, or by decoding it from
        base 64, depending on the binary attributes settings."""
        attrs = {}
        bad_attrs = []

        for name, value in self.module.params["attributes"].items():
            if self._is_binary(name):
                converter = base64.b64decode
            else:
                converter = to_bytes
            if not isinstance(value, list):
                value = [value]
            try:
                attrs[name] = [converter(v) for v in value]
            except binascii.Error:
                bad_attrs.append(name)

        return attrs, bad_attrs

    def add(self):
        """If self.dn does not exist, returns a callable that will add it."""

        def _add():
            self.connection.add_s(self.dn, modlist)

        if not self._is_entry_present():
            modlist = ldap.modlist.addModlist(self.attrs)
            action = _add
        else:
            action = None

        return action

    def delete(self):
        """If self.dn exists, returns a callable that will delete either
        the item itself if the recursive option is not set or the whole branch
        if it is."""

        def _delete():
            self.connection.delete_s(self.dn)

        def _delete_recursive():
            """Attempt recursive deletion using the subtree-delete control.
            If that fails, do it manually."""
            try:
                subtree_delete = ldap.controls.ValueLessRequestControl("1.2.840.113556.1.4.805")
                self.connection.delete_ext_s(self.dn, serverctrls=[subtree_delete])
            except ldap.NOT_ALLOWED_ON_NONLEAF:
                search = self.connection.search_s(self.dn, ldap.SCOPE_SUBTREE, attrlist=("dn",))
                search.reverse()
                for entry in search:
                    self.connection.delete_s(entry[0])

        if self._is_entry_present():
            if self.recursive:
                action = _delete_recursive
            else:
                action = _delete
        else:
            action = None

        return action

    def _is_entry_present(self):
        try:
            self.connection.search_s(self.dn, ldap.SCOPE_BASE)
        except ldap.NO_SUCH_OBJECT:
            is_present = False
        else:
            is_present = True

        return is_present


def main():
    module = AnsibleModule(
        argument_spec=gen_specs(
            attributes=dict(default={}, type="dict"),
            binary_attributes=dict(default=[], type="list", elements="str"),
            honor_binary=dict(default=False, type="bool"),
            objectClass=dict(type="list", elements="str"),
            state=dict(default="present", choices=["present", "absent"]),
            recursive=dict(default=False, type="bool"),
        ),
        required_if=[("state", "present", ["objectClass"])],
        supports_check_mode=True,
        required_together=ldap_required_together(),
    )

    if not HAS_LDAP:
        module.fail_json(msg=missing_required_lib("python-ldap"), exception=LDAP_IMP_ERR)

    state = module.params["state"]

    # Instantiate the LdapEntry object
    ldap = LdapEntry(module)

    # Get the action function
    if state == "present":
        action = ldap.add()
    elif state == "absent":
        action = ldap.delete()

    # Perform the action
    if action is not None and not module.check_mode:
        try:
            action()
        except Exception as e:
            module.fail_json(msg="Entry action failed.", details=f"{e}", exception=traceback.format_exc())

    module.exit_json(changed=(action is not None))


if __name__ == "__main__":
    main()
